<?php
 namespace App\Controller\Frontend; use Symfony\Component\HttpFoundation\Request; use Symfony\Component\HttpFoundation\Response; use Symfony\Component\Form\Form; use Symfony\Component\Form\Extension\Core\Type\SubmitType; use Symfony\Contracts\Translation\TranslatorInterface; use Symfony\Component\Validator\Validator\ValidatorInterface; use Symfony\Component\Validator\ConstraintViolationList; use App\Controller\Controller; use App\Event\EventQueue; use App\Service\Logger; use App\Entity\Site as SiteEntity; use App\Entity\Certificate as CertificateEntity; use App\Entity\NodejsSettings as NodejsSettingsEntity; use App\Entity\PhpSettings as PhpSettingsEntity; use App\Entity\PythonSettings as PythonSettingsEntity; use App\Entity\Database as DatabaseEntity; use App\Entity\DatabaseUser as DatabaseUserEntity; use App\Entity\DatabaseServer as DatabaseServerEntity; use App\Entity\User as UserEntity; use App\Entity\Manager\ConfigManager; use App\Entity\Manager\SiteManager as SiteEntityManager; use App\Entity\Manager\CertificateManager as CertificateEntityManager; use App\Entity\Manager\DatabaseManager as DatabaseEntityManager; use App\Entity\Manager\DatabaseUserManager as DatabaseUserEntityManager; use App\Entity\Manager\DatabaseServerManager as DatabaseServerEntityManager; use App\Entity\Manager\VhostTemplateManager as VhostTemplateEntityManager; use App\Database\Connection as DatabaseConnection; use App\Database\Manager as DatabaseManager; use App\Site\Nginx\Vhost\StaticTemplate as StaticVhostTemplate; use App\Site\Nginx\Vhost\PhpTemplate as PhpVhostTemplate; use App\Site\Nginx\Vhost\PythonTemplate as PythonVhostTemplate; use App\Site\Nginx\Vhost\ReverseProxyTemplate as ReverseProxyVhostTemplate; use App\Site\Nginx\Vhost\Processor\RedirectDomain as RedirectDomainProcessor; use App\Site\Nginx\Vhost\Processor\RedirectServerName as RedirectServerNameProcessor; use App\Site\Nginx\Vhost\Processor\ServerName as ServerNameProcessor; use App\Site\Parser\DomainName as DomainNameParser; use App\Site\Ssl\Generator\RsaKeyGenerator; use App\Site\Creator\NodejsSite as NodejsSiteCreator; use App\Site\Creator\StaticSite as StaticSiteCreator; use App\Site\Creator\PhpSite as PhpSiteCreator; use App\Site\Creator\PythonSite as PythonSiteCreator; use App\Site\Creator\ReverseProxySite as ReverseProxySiteCreator; use App\Site\Ssl\DistinguishedName; use App\Site\Ssl\Generator\CsrGenerator; use App\Site\Ssl\Util\Openssl; use App\Site\NodejsSite; use App\Site\StaticSite; use App\Site\PhpSite; use App\Site\PythonSite; use App\Site\ReverseProxySite; use App\Util\PasswordGenerator; use App\Site\Application\WordPressInstaller; use App\Service\Crypto; class NewSiteController extends Controller { private const WORDPRESS_VHOST_TEMPLATE_NAME = "\x57\x6f\x72\144\120\162\145\x73\163"; private const NODEJS_VHOST_TEMPLATE_NAME = "\x4e\157\x64\x65\x6a\163"; private const STATIC_SITE_VHOST_TEMPLATE_NAME = "\x53\x74\x61\x74\151\143"; private const PYTHON_VHOST_TEMPLATE_NAME = "\x50\171\x74\150\157\156"; private const REVERSE_PROXY_VHOST_TEMPLATE_NAME = "\x52\x65\166\x65\x72\163\x65\120\162\157\x78\171"; private const VARNISH_CACHE_SERVER = "\61\62\x37\x2e\x30\x2e\x30\56\x31\72\66\60\x38\61"; private const PASSWORD_LENGTH = 20; private ConfigManager $configManager; private SiteEntityManager $siteEntityManager; private CertificateEntityManager $certificateEntityManager; private DatabaseEntityManager $databaseEntityManager; private DatabaseUserEntityManager $databaseUserEntityManager; private DatabaseServerEntityManager $databaseServerEntityManager; private DomainNameParser $domainNameParser; private VhostTemplateEntityManager $vhostTemplateEntityManager; private ValidatorInterface $validator; private array $wordPressDefaultConfigValues = ["\x46\x53\x5f\115\x45\124\110\x4f\104" => ["\166\x61\x6c\165\145" => "\x64\151\162\x65\x63\x74", "\162\x61\x77" => false], "\127\120\x5f\x44\x45\x42\x55\107\x5f\x44\111\x53\120\114\x41\131" => ["\x76\x61\154\x75\x65" => "\146\x61\x6c\x73\145", "\x72\x61\167" => true], "\127\120\x5f\x44\x45\102\x55\x47\137\114\x4f\x47" => ["\166\x61\154\x75\145" => "\x74\x72\165\145", "\x72\x61\167" => true], "\x43\117\116\x43\101\124\x45\x4e\101\124\105\x5f\123\x43\122\111\x50\124\x53" => ["\166\141\x6c\165\145" => "\x66\x61\x6c\x73\x65", "\x72\141\x77" => true], "\101\x55\124\x4f\x53\x41\x56\105\x5f\111\116\x54\x45\x52\126\101\x4c" => ["\166\x61\x6c\165\x65" => 600, "\x72\141\x77" => true], "\x57\120\137\120\117\123\x54\137\x52\105\126\x49\x53\x49\117\116\123" => ["\x76\141\154\x75\145" => 5, "\162\x61\x77" => true], "\x45\115\x50\124\131\x5f\x54\122\101\x53\110\x5f\104\x41\131\123" => ["\166\141\154\165\145" => 21, "\162\x61\x77" => true]]; private array $wordPressMultiSiteConfigValues = ["\x57\120\x5f\x41\x4c\x4c\x4f\x57\x5f\115\x55\114\x54\111\123\111\x54\105" => ["\x76\x61\154\165\x65" => "\x74\162\165\x65", "\x72\x61\167" => true], "\115\x55\114\124\x49\x53\111\x54\105" => ["\x76\x61\x6c\x75\x65" => "\164\x72\x75\145", "\162\141\x77" => true], "\123\125\102\104\x4f\x4d\x41\x49\x4e\137\111\x4e\x53\124\x41\x4c\114" => ["\x76\x61\154\165\x65" => "\x66\141\x6c\x73\x65", "\162\141\167" => true], "\104\117\115\101\111\x4e\137\x43\125\122\122\x45\116\124\137\123\111\x54\105" => ["\166\141\x6c\165\x65" => '', "\162\x61\x77" => false], "\120\x41\124\x48\x5f\x43\125\x52\x52\105\x4e\x54\137\123\x49\124\x45" => ["\x76\141\154\x75\145" => "\57", "\162\x61\x77" => false], "\123\x49\x54\105\137\111\104\x5f\x43\x55\x52\122\105\x4e\124\x5f\123\111\124\105" => ["\x76\141\154\165\x65" => 1, "\x72\x61\x77" => true], "\102\114\117\x47\x5f\x49\x44\137\103\125\x52\122\x45\x4e\124\137\123\x49\124\105" => ["\x76\141\154\165\x65" => 1, "\x72\x61\x77" => true]]; public function __construct(ConfigManager $configManager, SiteEntityManager $siteEntityManager, CertificateEntityManager $certificateEntityManager, DatabaseEntityManager $databaseEntityManager, DatabaseUserEntityManager $databaseUserEntityManager, DatabaseServerEntityManager $databaseServerEntityManager, VhostTemplateEntityManager $vhostTemplateEntityManager, DomainNameParser $domainNameParser, ValidatorInterface $validator, TranslatorInterface $translator, Logger $logger) { goto F7177; b9ae8: $this->databaseServerEntityManager = $databaseServerEntityManager; goto a310d; a310d: $this->vhostTemplateEntityManager = $vhostTemplateEntityManager; goto ff44f; cb6aa: $this->validator = $validator; goto D8051; a7627: $this->databaseUserEntityManager = $databaseUserEntityManager; goto b9ae8; D8051: parent::__construct($translator, $logger); goto ab247; F7177: $this->configManager = $configManager; goto B037f; e3e11: $this->databaseEntityManager = $databaseEntityManager; goto a7627; ff44f: $this->domainNameParser = $domainNameParser; goto cb6aa; c2c8b: $this->certificateEntityManager = $certificateEntityManager; goto e3e11; B037f: $this->siteEntityManager = $siteEntityManager; goto c2c8b; ab247: } public function index(Request $request) : Response { goto Fcb74; c8b72: $response = $this->render("\x46\162\157\156\164\145\156\144\57\123\151\164\145\x2f\x4e\145\x77\x2f\151\x6e\x64\x65\x78\x2e\150\x74\155\154\x2e\164\167\x69\147", ["\165\163\145\162" => $user]); goto D1392; Fcb74: $hasPermissions = $this->hasPermissions(); goto ad436; D1392: a07c6: goto eec27; Ce468: $user = $this->getUser(); goto c8b72; Da34d: goto a07c6; goto a3699; eec27: return $response; goto c9566; a3699: ea627: goto Ce468; F36a7: $response = $this->redirect($this->generateUrl("\143\154\160\137\163\151\x74\x65\x73")); goto Da34d; ad436: if (true === $hasPermissions) { goto ea627; } goto F36a7; c9566: } public function newPythonSite(Request $request) : Response { goto f50a0; a0a42: $user = $this->getUser(); goto aa507; f50a0: $hasPermissions = $this->hasPermissions(); goto b03ab; Aa816: $response = $this->render("\x46\162\x6f\x6e\164\x65\x6e\144\x2f\x53\151\164\x65\x2f\116\145\x77\57\x70\171\164\150\157\x6e\x2e\150\x74\x6d\154\56\164\x77\151\x67", ["\x75\x73\145\x72" => $user, "\x66\x6f\162\155" => $form->createView(), "\x66\157\162\155\105\x72\162\x6f\x72\x73" => $this->formErrors]); goto ceecf; B67fa: $response = $this->redirect($this->generateUrl("\143\x6c\160\x5f\x73\x69\x74\x65\163")); goto b2435; cf4a2: e64d5: goto a0a42; d23a0: $form = $this->createPythonSiteForm($pythonSettings); goto D0b0b; D4af0: if (!(false === is_null($response))) { goto c02f4; } goto ed2bd; d83c5: $form->handleRequest($request); goto D38d7; D38d7: if (!(true === $form->isSubmitted())) { goto a7065; } goto b158c; b158c: $response = $this->handleCreatePythonSiteForm($request, $form); goto D4af0; b2435: goto C5f9c; goto cf4a2; fb585: return $response; goto Fa04e; cf6f7: c02f4: goto f8196; D0b0b: if (!(true === $request->isMethod("\120\x4f\x53\x54"))) { goto b3678; } goto d83c5; ed2bd: return $response; goto cf6f7; ceecf: C5f9c: goto fb585; c5cf6: b3678: goto Aa816; f8196: a7065: goto c5cf6; b03ab: if (true === $hasPermissions) { goto e64d5; } goto B67fa; aa507: $pythonSettings = new PythonSettingsEntity(); goto d23a0; Fa04e: } private function createPythonSiteForm(PythonSettingsEntity $pythonSettings) : Form { goto c1def; f4d89: return $form; goto b484e; c1def: $form = $this->createForm("\101\x70\160\134\106\157\162\155\134\123\151\164\145\x4e\145\x77\120\x79\x74\x68\157\156\x54\x79\x70\x65", $pythonSettings, ["\x61\x63\x74\x69\x6f\x6e" => $this->generateUrl("\x63\154\x70\x5f\163\151\164\145\137\x70\171\164\x68\x6f\x6e\137\x6e\x65\167"), "\x6d\x65\164\x68\157\x64" => "\x50\117\123\x54", "\x61\x74\x74\x72" => ["\x69\x64" => "\x6e\145\x77\55\x70\171\164\150\x6f\x6e\55\163\x69\164\x65\55\146\157\162\155"]]); goto C9996; C9996: $form->add("\x73\x75\142\155\151\164", SubmitType::class, ["\x61\164\164\x72" => ["\143\x6c\141\163\163" => "\x62\164\x6e\x20\x62\164\156\x2d\x6c\147\x20\142\164\156\x2d\x62\154\165\x65"], "\x6c\x61\142\145\x6c" => "\x43\x72\x65\x61\x74\145"]); goto f4d89; b484e: } private function handleCreatePythonSiteForm(Request $request, Form $form) { goto Eafa6; f6a26: goto A19dd; goto D9bfb; Eeb92: $this->formErrors = $this->getErrorMessages($form); goto f6a26; a76c4: try { goto Bbe53; F2b3e: if (false === is_null($vhostTemplateEntity)) { goto Cbc5a; } goto efb02; F901b: $this->createPythonSite($domainName, $pythonVersion, $port, $siteUser, $siteUserPassword); goto c0d58; B3ae1: EventQueue::addEvent(EventQueue::EVENT_SITE_PYTHON_CREATE, $user, $eventData, $request); goto f0f32; f60c2: $domainName = $form->get("\x64\x6f\x6d\x61\x69\156\116\141\155\x65")->getData(); goto b0c3d; efb02: throw new \Exception(sprintf("\x56\150\157\x73\164\40\x74\145\155\160\154\141\x74\145\40\42\45\x73\42\40\156\x6f\164\x20\x66\x6f\165\156\144\x2e", $vhostTemplateName)); goto Cb617; e31ea: $siteUser = $form->get("\x73\x69\x74\145\125\x73\145\x72")->getData(); goto a53da; a1253: $vhostTemplateName = self::PYTHON_VHOST_TEMPLATE_NAME; goto b6546; Cb617: goto A3b99; goto F33ef; b0c3d: $pythonVersion = $form->get("\160\171\x74\x68\157\156\x56\x65\x72\x73\151\x6f\x6e")->getData(); goto Ed898; f0f32: $session->getFlashBag()->set("\x73\x75\143\143\145\x73\x73", $this->translator->trans("\123\151\164\145\x20\150\x61\x73\40\142\145\x65\156\40\x63\162\x65\x61\164\x65\x64\x2e")); goto d1772; Bbe53: $session = $request->getSession(); goto f60c2; Ed898: $port = $form->get("\160\157\162\164")->getData(); goto e31ea; Bf3fc: A3b99: goto de586; c0d58: $eventData = ["\163\151\164\x65" => $domainName, "\163\x69\164\145\125\x73\145\x72" => $siteUser, "\160\x79\164\x68\157\156\x56\145\x72\x73\151\x6f\x6e" => $pythonVersion, "\160\157\x72\x74" => $port]; goto B3ae1; b6546: $vhostTemplateEntity = $this->getVhostTemplateEntity($vhostTemplateName); goto F2b3e; Fa15c: $user = $this->getUser(); goto F901b; d1772: $response = $this->redirect($this->generateUrl("\143\x6c\160\137\163\151\164\145\163")); goto a2ac5; F33ef: Cbc5a: goto Fa15c; a2ac5: return $response; goto Bf3fc; a53da: $siteUserPassword = $form->get("\x73\x69\164\145\x55\163\x65\162\120\141\163\163\x77\157\162\x64")->getData(); goto a1253; de586: } catch (\Exception $e) { $this->logger->exception($e); $session->getFlashBag()->set("\x64\x61\156\x67\145\x72", $this->translator->trans("\x41\156\x20\x65\x72\162\157\162\40\150\141\163\40\x6f\143\x63\x75\162\162\x65\x64\54\x20\x65\x72\x72\x6f\x72\40\x6d\x65\163\163\141\x67\x65\x3a\40\45\145\x72\x72\157\x72\115\x65\163\163\141\x67\x65\x25", ["\x25\x65\162\162\157\162\115\145\163\163\141\147\145\x25" => $e->getMessage()])); } goto Bbb44; D9bfb: A1d0f: goto a76c4; Bbb44: A19dd: goto f8379; Eafa6: if (true === $form->isValid()) { goto A1d0f; } goto Eeb92; f8379: } public function newReverseProxy(Request $request) : Response { goto d7add; e5e4f: Ae147: goto fdc78; ef94f: if (!(true === $form->isSubmitted())) { goto C4a9e; } goto f40f1; Acaed: Ea217: goto F8179; c6320: $form->handleRequest($request); goto ef94f; Faf34: if (!(false === is_null($response))) { goto Ae147; } goto C5a8d; d7add: $hasPermissions = $this->hasPermissions(); goto E9f6a; Fdc69: F6ca3: goto Ad7dc; f40f1: $response = $this->handleReverseProxyForm($request, $form); goto Faf34; E9f79: goto Ea217; goto c67f5; F8179: return $response; goto Baf55; F99a8: $response = $this->redirect($this->generateUrl("\143\154\160\x5f\x73\x69\164\x65\163")); goto E9f79; a216c: $form = $this->createReverseProxyForm(); goto efc25; B235b: $user = $this->getUser(); goto a216c; C5a8d: return $response; goto e5e4f; Ad7dc: $response = $this->render("\x46\x72\x6f\156\164\x65\x6e\144\57\x53\x69\x74\x65\x2f\x4e\x65\x77\x2f\162\x65\166\145\162\163\x65\x2d\160\x72\x6f\170\x79\56\150\164\155\x6c\56\x74\167\x69\147", ["\165\x73\x65\x72" => $user, "\146\157\x72\155" => $form->createView(), "\x66\157\x72\x6d\x45\162\x72\x6f\162\163" => $this->formErrors]); goto Acaed; c67f5: C253c: goto B235b; efc25: if (!(true === $request->isMethod("\120\x4f\123\124"))) { goto F6ca3; } goto c6320; fdc78: C4a9e: goto Fdc69; E9f6a: if (true === $hasPermissions) { goto C253c; } goto F99a8; Baf55: } private function createReverseProxyForm() : Form { goto d8399; F544f: $form->add("\163\165\142\155\151\x74", SubmitType::class, ["\x61\164\x74\x72" => ["\x63\x6c\x61\163\163" => "\x62\164\156\40\142\x74\156\55\x6c\147\40\142\x74\x6e\x2d\142\154\165\x65"], "\154\141\x62\x65\x6c" => "\x43\162\x65\141\164\145"]); goto be3f7; be3f7: return $form; goto ca17c; d8399: $form = $this->createForm("\101\160\160\x5c\106\x6f\162\x6d\134\x53\151\164\145\116\145\x77\122\145\166\x65\x72\163\145\x50\162\157\x78\x79\124\171\x70\x65", [], ["\x61\x63\164\151\x6f\156" => $this->generateUrl("\x63\154\x70\137\163\x69\164\145\137\x72\145\166\145\x72\163\145\137\160\x72\x6f\x78\x79\137\156\145\x77"), "\x6d\145\164\150\x6f\x64" => "\x50\x4f\x53\x54", "\141\x74\164\162" => ["\x69\x64" => "\156\145\x77\x2d\162\145\x76\x65\x72\163\x65\55\x70\162\x6f\170\x79\55\146\x6f\x72\x6d"]]); goto F544f; ca17c: } private function handleReverseProxyForm(Request $request, Form $form) { goto d9095; C89a4: $this->formErrors = $this->getErrorMessages($form); goto E43cd; b5816: A707a: goto f8d40; E43cd: goto A707a; goto E99dd; d9095: if (true === $form->isValid()) { goto D1d56; } goto C89a4; efb19: try { goto C8db8; F1713: $domainName = $form->get("\144\157\x6d\x61\x69\156\116\x61\x6d\145")->getData(); goto Efdac; c6643: throw new \Exception(sprintf("\126\150\x6f\x73\164\x20\164\145\x6d\x70\154\141\164\145\x20\x22\45\163\x22\x20\156\157\164\x20\x66\x6f\165\x6e\x64\56", $vhostTemplateName)); goto A2591; Dd791: $siteUserPassword = $form->get("\163\151\x74\x65\x55\x73\x65\x72\x50\x61\x73\x73\167\x6f\162\144")->getData(); goto A61ba; A2591: goto F19f9; goto D64bb; A6e11: $this->createReverseProxySite($domainName, $reverseProxyUrl, $siteUser, $siteUserPassword); goto dffd7; fb280: $vhostTemplateName = self::REVERSE_PROXY_VHOST_TEMPLATE_NAME; goto da8ac; dffd7: $eventData = ["\163\151\x74\145" => $domainName, "\x73\x69\x74\x65\125\x73\x65\x72" => $siteUser, "\x72\145\x76\145\162\x73\x65\x50\162\157\x78\171\x55\x72\x6c" => $reverseProxyUrl]; goto c9f59; fdbd6: $session->getFlashBag()->set("\163\165\x63\x63\x65\x73\163", $this->translator->trans("\122\145\166\145\x72\163\145\40\120\x72\x6f\170\171\40\x68\141\x73\40\142\145\x65\x6e\x20\x63\x72\145\x61\164\x65\144\56")); goto e5650; e5650: $response = $this->redirect($this->generateUrl("\x63\154\160\137\x73\x69\x74\x65\163")); goto aa4c8; da8ac: $vhostTemplateEntity = $this->getVhostTemplateEntity($vhostTemplateName); goto F466a; aa4c8: return $response; goto B82e4; D64bb: a1991: goto f76fb; c9f59: EventQueue::addEvent(EventQueue::EVENT_SITE_REVERSE_PROXY_CREATE, $user, $eventData, $request); goto fdbd6; C8db8: $session = $request->getSession(); goto F1713; A61ba: $reverseProxyUrl = $form->get("\x72\145\x76\x65\162\163\145\120\162\157\x78\x79\125\x72\154")->getData(); goto fb280; Efdac: $siteUser = $form->get("\163\151\164\145\125\163\x65\x72")->getData(); goto Dd791; F466a: if (false === is_null($vhostTemplateEntity)) { goto a1991; } goto c6643; f76fb: $user = $this->getUser(); goto A6e11; B82e4: F19f9: goto Bf048; Bf048: } catch (\Exception $e) { $this->logger->exception($e); $session->getFlashBag()->set("\144\141\x6e\147\x65\x72", $this->translator->trans("\101\156\40\x65\162\162\157\162\x20\x68\x61\163\x20\157\143\143\x75\162\x72\145\x64\54\x20\x65\x72\x72\157\x72\40\155\x65\163\163\141\147\145\x3a\x20\45\145\162\162\157\x72\115\x65\163\x73\x61\147\145\45", ["\45\145\162\x72\157\162\115\145\x73\163\141\147\x65\x25" => $e->getMessage()])); } goto b5816; E99dd: D1d56: goto efb19; f8d40: } public function newNodejsSite(Request $request) : Response { goto Cc82c; ff656: Fdf57: goto Db066; Be4d8: if (!(false === is_null($response))) { goto Fdf57; } goto d465f; c2e81: e7ddd: goto ffeb6; df36c: D19b2: goto d4368; B4725: $form = $this->createNodejsSiteForm($nodejsSettings); goto D836f; D836f: if (!(true === $request->isMethod("\120\117\123\x54"))) { goto e7ddd; } goto ebfcb; C2406: return $response; goto Ee665; a123e: if (!(true === $form->isSubmitted())) { goto A7029; } goto Cf679; Cf679: $response = $this->handleCreateNodejsSiteForm($request, $form); goto Be4d8; f8215: $response = $this->redirect($this->generateUrl("\x63\154\160\137\x73\151\x74\145\x73")); goto A2597; Db066: A7029: goto c2e81; A6fdc: C5b47: goto C2406; Cc82c: $hasPermissions = $this->hasPermissions(); goto F791a; A2597: goto C5b47; goto df36c; ffeb6: $response = $this->render("\106\162\x6f\156\164\145\156\x64\57\x53\x69\x74\145\57\116\x65\x77\x2f\x6e\x6f\144\x65\x6a\x73\56\x68\x74\x6d\x6c\56\x74\167\x69\147", ["\x75\163\x65\x72" => $user, "\146\x6f\x72\x6d" => $form->createView(), "\x66\157\162\155\105\x72\162\x6f\162\163" => $this->formErrors]); goto A6fdc; B9f68: $nodejsSettings = new NodejsSettingsEntity(); goto B4725; F791a: if (true === $hasPermissions) { goto D19b2; } goto f8215; d4368: $user = $this->getUser(); goto B9f68; ebfcb: $form->handleRequest($request); goto a123e; d465f: return $response; goto ff656; Ee665: } private function createNodejsSiteForm(NodejsSettingsEntity $nodejsSettings) : Form { goto Fea7f; ff474: $form->add("\163\x75\x62\155\151\164", SubmitType::class, ["\x61\x74\164\162" => ["\x63\x6c\x61\163\163" => "\142\x74\156\40\x62\164\156\55\154\x67\x20\142\164\156\x2d\x62\154\x75\145"], "\154\x61\142\x65\154" => "\103\x72\x65\x61\164\x65"]); goto Dd5a1; Fea7f: $form = $this->createForm("\x41\x70\160\x5c\106\157\162\155\x5c\x53\151\164\x65\x4e\145\x77\116\157\144\x65\x6a\163\x54\171\160\145", $nodejsSettings, ["\141\x63\164\151\x6f\156" => $this->generateUrl("\x63\154\x70\x5f\x73\x69\164\x65\137\x6e\x6f\144\145\152\163\x5f\x6e\x65\167"), "\x6d\145\x74\150\x6f\144" => "\x50\117\x53\x54", "\141\x74\x74\162" => ["\151\x64" => "\x6e\145\167\x2d\x6e\x6f\x64\x65\x6a\163\x2d\x73\x69\x74\145\55\x66\157\162\155"]]); goto ff474; Dd5a1: return $form; goto Dad8c; Dad8c: } private function handleCreateNodejsSiteForm(Request $request, Form $form) { goto f2327; Cb71f: goto C966b; goto D5bae; a11c9: C966b: goto b178e; D5bae: e85bb: goto aa016; f2327: if (true === $form->isValid()) { goto e85bb; } goto c17c5; c17c5: $this->formErrors = $this->getErrorMessages($form); goto Cb71f; aa016: try { goto F11fd; C3338: $siteUser = $form->get("\x73\151\164\145\x55\163\x65\162")->getData(); goto B8a39; Cb651: return $response; goto Dddd2; d778e: $response = $this->redirect($this->generateUrl("\143\x6c\x70\x5f\x73\151\x74\x65\x73")); goto Cb651; c22c4: EventQueue::addEvent(EventQueue::EVENT_SITE_NODEJS_CREATE, $user, $eventData, $request); goto dbed8; dbed8: $session->getFlashBag()->set("\x73\x75\x63\143\145\x73\x73", $this->translator->trans("\x53\151\164\x65\x20\x68\x61\x73\x20\x62\x65\145\x6e\x20\x63\x72\x65\141\x74\x65\144\x2e")); goto d778e; D9c38: $vhostTemplateName = self::NODEJS_VHOST_TEMPLATE_NAME; goto Fc521; Fc521: $vhostTemplateEntity = $this->getVhostTemplateEntity($vhostTemplateName); goto a4262; f5db9: $domainName = $form->get("\x64\157\155\141\x69\156\x4e\x61\x6d\x65")->getData(); goto B95c5; Dbe4a: Bc415: goto A4858; e729f: goto ee302; goto Dbe4a; c457f: throw new \Exception(sprintf("\x56\x68\157\163\x74\40\164\x65\155\160\154\141\164\145\x20\42\45\163\42\x20\156\x6f\164\40\146\x6f\x75\x6e\144\56", $vhostTemplateName)); goto e729f; B8a39: $siteUserPassword = $form->get("\163\151\164\x65\x55\x73\x65\x72\120\141\163\x73\167\x6f\162\x64")->getData(); goto D9c38; a4262: if (false === is_null($vhostTemplateEntity)) { goto Bc415; } goto c457f; c9be3: $eventData = ["\x73\151\x74\x65" => $domainName, "\163\x69\x74\x65\x55\x73\x65\162" => $siteUser, "\156\x6f\x64\145\152\x73\x56\x65\162\163\151\157\x6e" => $nodejsVersion, "\160\x6f\x72\x74" => $port]; goto c22c4; f8beb: $port = $form->get("\x70\x6f\x72\164")->getData(); goto C3338; Dddd2: ee302: goto c84c5; B95c5: $nodejsVersion = $form->get("\156\157\144\x65\152\x73\x56\x65\162\163\151\x6f\x6e")->getData(); goto f8beb; A4858: $user = $this->getUser(); goto ef868; F11fd: $session = $request->getSession(); goto f5db9; ef868: $this->createNodejsSite($domainName, $nodejsVersion, $port, $siteUser, $siteUserPassword); goto c9be3; c84c5: } catch (\Exception $e) { $this->logger->exception($e); $session->getFlashBag()->set("\x64\x61\x6e\147\145\x72", $this->translator->trans("\101\156\x20\x65\x72\162\157\162\x20\150\141\163\40\x6f\x63\x63\165\x72\x72\x65\144\x2c\x20\145\x72\x72\157\x72\40\155\x65\x73\x73\141\x67\145\72\x20\45\145\162\x72\157\x72\x4d\x65\x73\x73\141\x67\x65\x25", ["\x25\145\162\x72\157\162\x4d\x65\163\x73\141\147\x65\45" => $e->getMessage()])); } goto a11c9; b178e: } public function newStaticSite(Request $request) : Response { goto abdfc; bf3bc: goto Fc8e8; goto d0e71; afdfb: Fc8e8: goto Caddf; abdfc: $hasPermissions = $this->hasPermissions(); goto Bb18e; a0b45: $response = $this->redirect($this->generateUrl("\x63\x6c\x70\137\163\x69\x74\145\163")); goto bf3bc; Ae39a: $response = $this->handleCreateStaticSiteForm($request, $form); goto Fda6f; Caddf: return $response; goto ce185; fd75b: bf87b: goto b1e88; b1e88: $response = $this->render("\x46\162\x6f\156\164\145\156\144\x2f\x53\x69\164\145\x2f\x4e\x65\x77\x2f\x73\x74\141\164\151\143\56\150\164\x6d\x6c\x2e\x74\x77\151\x67", ["\x75\x73\145\x72" => $user, "\146\x6f\x72\x6d" => $form->createView(), "\x66\x6f\x72\155\x45\162\x72\x6f\x72\x73" => $this->formErrors]); goto afdfb; d0e71: cf59d: goto a2470; b07d5: $form->handleRequest($request); goto a88c0; a94e8: return $response; goto Ac6f4; Fda6f: if (!(false === is_null($response))) { goto Ae863; } goto a94e8; Af5db: $form = $this->createStaticSiteForm(); goto Ad154; a88c0: if (!(true === $form->isSubmitted())) { goto D63e9; } goto Ae39a; a2470: $user = $this->getUser(); goto Af5db; Bf0fe: D63e9: goto fd75b; Ac6f4: Ae863: goto Bf0fe; Ad154: if (!(true === $request->isMethod("\120\117\123\124"))) { goto bf87b; } goto b07d5; Bb18e: if (true === $hasPermissions) { goto cf59d; } goto a0b45; ce185: } private function createStaticSiteForm() : Form { goto D49ce; D49ce: $form = $this->createForm("\x41\x70\x70\134\x46\157\x72\155\134\123\x69\164\x65\x4e\145\x77\123\164\x61\x74\x69\x63\124\x79\x70\x65", [], ["\141\x63\x74\x69\x6f\x6e" => $this->generateUrl("\x63\x6c\x70\137\x73\151\x74\x65\x5f\x73\x74\x61\x74\x69\x63\x5f\156\x65\167"), "\155\x65\164\x68\157\x64" => "\120\x4f\123\x54", "\x61\164\164\x72" => []]); goto a4f1e; B1ae5: return $form; goto Bf2aa; a4f1e: $form->add("\x73\x75\x62\x6d\151\x74", SubmitType::class, ["\x61\x74\x74\x72" => ["\x63\154\141\x73\163" => "\142\164\x6e\40\x62\x74\156\55\154\147\x20\142\x74\156\x2d\142\x6c\x75\x65"], "\x6c\141\142\x65\154" => "\103\x72\145\141\164\145"]); goto B1ae5; Bf2aa: } private function handleCreateStaticSiteForm(Request $request, Form $form) { goto E5da0; ddd88: try { goto df20f; e0a81: $vhostTemplateName = self::STATIC_SITE_VHOST_TEMPLATE_NAME; goto a911c; f108c: $domainName = $form->get("\x64\x6f\155\x61\x69\x6e\116\141\x6d\145")->getData(); goto Ecf7d; eb87c: $session->getFlashBag()->set("\x73\x75\143\x63\145\x73\x73", $this->translator->trans("\123\151\164\x65\40\x68\x61\163\x20\x62\145\x65\x6e\x20\x63\162\x65\x61\x74\145\x64\56")); goto af361; Fc106: $user = $this->getUser(); goto Dc48f; aa7f1: throw new \Exception(sprintf("\126\150\x6f\x73\164\x20\164\145\x6d\160\x6c\x61\x74\x65\x20\x22\45\x73\x22\40\156\157\x74\x20\x66\157\x75\x6e\144\x2e", $vhostTemplateName)); goto Cf661; Cf661: goto d7926; goto C522a; B8150: EventQueue::addEvent(EventQueue::EVENT_SITE_STATIC_CREATE, $user, $eventData, $request); goto eb87c; C522a: B3b52: goto Fc106; Cca74: $eventData = ["\x73\151\x74\x65" => $domainName, "\x73\151\164\145\x55\163\x65\162" => $siteUser]; goto B8150; Dc48f: $this->createStaticSite($domainName, $siteUser, $siteUserPassword); goto Cca74; dd93e: $siteUserPassword = $form->get("\x73\x69\164\x65\x55\163\145\x72\x50\141\x73\163\x77\157\x72\x64")->getData(); goto e0a81; a911c: $vhostTemplateEntity = $this->getVhostTemplateEntity($vhostTemplateName); goto aa01c; af361: $response = $this->redirect($this->generateUrl("\143\154\x70\137\163\151\164\x65\163")); goto F3181; Ecf7d: $siteUser = $form->get("\x73\x69\164\x65\125\x73\x65\x72")->getData(); goto dd93e; F3181: return $response; goto Cc304; Cc304: d7926: goto F59d1; aa01c: if (false === is_null($vhostTemplateEntity)) { goto B3b52; } goto aa7f1; df20f: $session = $request->getSession(); goto f108c; F59d1: } catch (\Exception $e) { $this->logger->exception($e); $session->getFlashBag()->set("\x64\141\156\x67\145\162", $this->translator->trans("\x41\x6e\x20\145\x72\x72\x6f\x72\x20\150\x61\x73\x20\x6f\x63\x63\165\162\162\x65\x64\54\40\145\x72\x72\157\162\x20\x6d\x65\163\x73\x61\x67\x65\72\40\x25\x65\162\162\x6f\x72\x4d\x65\x73\163\141\147\145\x25", ["\45\x65\x72\162\157\x72\115\145\163\x73\141\147\145\45" => $e->getMessage()])); } goto Fe38f; ab506: F91d5: goto ddd88; Fe38f: dfa89: goto Ab99b; Dc6f7: $this->formErrors = $this->getErrorMessages($form); goto E4b05; E5da0: if (true === $form->isValid()) { goto F91d5; } goto Dc6f7; E4b05: goto dfa89; goto ab506; Ab99b: } public function newPhpSite(Request $request) : Response { goto Befe8; a89a8: if (!(true === $request->isMethod("\120\117\123\x54"))) { goto cf283; } goto E5b6d; E5b6d: $form->handleRequest($request); goto c777e; fad0d: a7943: goto Dfd07; Dfd07: cf283: goto e69f1; a7694: f9bd8: goto faf72; B5e5b: if (true === $hasPermissions) { goto f9bd8; } goto D3422; f90e7: return $response; goto F3cad; A3439: b7aef: goto Bc4ee; Bc4ee: return $response; goto e365b; e69f1: $response = $this->render("\106\162\157\156\x74\145\156\144\57\123\x69\x74\145\x2f\x4e\x65\167\57\x70\x68\x70\x2e\x68\x74\155\x6c\x2e\x74\x77\x69\147", ["\x75\x73\145\162" => $user, "\146\x6f\x72\x6d" => $form->createView(), "\146\x6f\x72\x6d\105\x72\162\x6f\x72\x73" => $this->formErrors]); goto A3439; a4696: $response = $this->handleCreatePhpSiteForm($request, $form); goto e848e; faf72: $user = $this->getUser(); goto Fd8f8; D3422: $response = $this->redirect($this->generateUrl("\x63\x6c\160\x5f\x73\151\164\145\x73")); goto Dd021; Fd8f8: $form = $this->createPhpSiteForm(); goto a89a8; F3cad: Cfa6f: goto fad0d; c777e: if (!(true === $form->isSubmitted())) { goto a7943; } goto a4696; e848e: if (!(false === is_null($response))) { goto Cfa6f; } goto f90e7; Befe8: $hasPermissions = $this->hasPermissions(); goto B5e5b; Dd021: goto b7aef; goto a7694; e365b: } private function createPhpSiteForm() : Form { goto bcab2; Ea81b: $form->add("\x73\x75\x62\x6d\x69\x74", SubmitType::class, ["\141\x74\164\x72" => ["\143\154\x61\x73\x73" => "\x62\164\156\40\x62\164\x6e\55\x6c\147\x20\x62\164\156\55\x62\x6c\x75\x65"], "\x6c\141\x62\x65\154" => "\103\x72\145\141\x74\145"]); goto ba8fb; ba8fb: return $form; goto Bae1a; bcab2: $form = $this->createForm("\101\160\160\134\x46\157\x72\155\134\x53\x69\x74\145\x4e\145\x77\x50\150\160\124\171\160\x65", [], ["\141\143\x74\151\x6f\156" => $this->generateUrl("\143\x6c\x70\x5f\x73\151\x74\145\x5f\x70\x68\160\x5f\x6e\145\x77"), "\x6d\x65\164\150\x6f\144" => "\120\117\x53\x54", "\141\164\164\x72" => []]); goto Ea81b; Bae1a: } private function handleCreatePhpSiteForm(Request $request, Form $form) { goto ec069; a5ac0: try { goto a6bc8; a6bc8: $session = $request->getSession(); goto D8ffb; e972f: $phpVersion = $form->get("\x70\150\x70\126\x65\x72\163\151\157\x6e")->getData(); goto D3ba4; D3ba4: $siteUser = $form->get("\x73\151\164\x65\125\163\145\x72")->getData(); goto b852b; A6611: $user = $this->getUser(); goto c946a; e44b4: $varnishCacheSettings = []; goto F52b2; A5907: $this->createPhpSite($domainName, $rootDirectory, $phpVersion, $vhostTemplateName, $siteUser, $siteUserPassword, $varnishCacheSettings); goto d6446; d6446: $eventData = ["\x73\x69\164\145" => $domainName, "\x76\150\x6f\163\x74\124\x65\155\x70\x6c\x61\164\145\x4e\141\x6d\145" => $vhostTemplateName, "\160\150\x70\x56\145\x72\x73\x69\157\x6e" => $phpVersion, "\x73\151\x74\145\125\x73\145\162" => $siteUser]; goto d645a; F52b2: if (!(false === empty($vhostTemplateEntity->getVarnishCacheSettings()))) { goto c013c; } goto B368c; B368c: $varnishCacheSettings = (array) json_decode($vhostTemplateEntity->getVarnishCacheSettings(), true); goto e8ed7; Ac885: throw new \Exception(sprintf("\x56\x68\157\x73\x74\40\x74\145\155\160\x6c\x61\x74\x65\x20\42\45\163\42\x20\156\157\164\40\x66\x6f\165\x6e\144\56", $vhostTemplateName)); goto bcf93; e8ed7: c013c: goto A5907; A27f0: return $response; goto B04df; e38da: if (false === is_null($vhostTemplateEntity)) { goto Abbd5; } goto Ac885; b852b: $siteUserPassword = $form->get("\163\151\x74\145\x55\163\x65\x72\x50\x61\x73\163\x77\157\162\x64")->getData(); goto B19f8; B19f8: $vhostTemplateEntity = $this->getVhostTemplateEntity($vhostTemplateName); goto e38da; Cb52d: $domainName = $form->get("\x64\x6f\x6d\141\x69\x6e\116\141\x6d\145")->getData(); goto e972f; d645a: EventQueue::addEvent(EventQueue::EVENT_SITE_PHP_CREATE, $user, $eventData, $request); goto a752f; a752f: $session->getFlashBag()->set("\x73\x75\x63\x63\145\163\163", $this->translator->trans("\123\151\x74\145\40\x68\141\163\x20\142\145\x65\156\40\x63\x72\x65\141\x74\x65\144\x2e")); goto cc7b4; B04df: c0178: goto ad9bb; cc7b4: $response = $this->redirect($this->generateUrl("\x63\154\160\137\x73\x69\x74\x65\x73")); goto A27f0; bcf93: goto c0178; goto cffd3; b61b4: $rootDirectory = sprintf("\45\x73\x2f\45\x73", $rootDirectory, ltrim(rtrim($vhostTemplateEntity->getRootDirectory(), "\x2f"), "\57")); goto Ea025; Ea025: Caa6d: goto e44b4; D8ffb: $vhostTemplateName = $form->get("\x61\x70\x70\x6c\151\x63\141\164\151\x6f\x6e")->getData(); goto Cb52d; Ab37d: if (!(false === empty($vhostTemplateEntity->getRootDirectory()))) { goto Caa6d; } goto b61b4; c946a: $rootDirectory = $domainName; goto Ab37d; cffd3: Abbd5: goto A6611; ad9bb: } catch (\Exception $e) { $this->logger->exception($e); $session->getFlashBag()->set("\144\141\x6e\147\x65\x72", $this->translator->trans("\x41\156\x20\145\x72\162\x6f\x72\x20\x68\x61\163\40\157\x63\143\x75\x72\162\x65\144\54\40\145\x72\x72\157\162\40\x6d\145\163\163\x61\147\145\72\40\x25\145\x72\x72\x6f\x72\x4d\145\163\163\x61\x67\145\45", ["\x25\x65\x72\162\x6f\162\x4d\x65\x73\x73\141\147\x65\45" => $e->getMessage()])); } goto f566f; D7cc8: $this->formErrors = $this->getErrorMessages($form); goto Dad09; Dad09: goto Fefb6; goto B161b; ec069: if (true === $form->isValid()) { goto C2474; } goto D7cc8; f566f: Fefb6: goto c1e0b; B161b: C2474: goto a5ac0; c1e0b: } public function newWordPressSite(Request $request) : Response { goto ca7bd; ca7bd: $hasPermissions = $this->hasPermissions(); goto d69fb; D27a6: Ed890: goto aa73b; D6a27: $response = $this->redirect($this->generateUrl("\143\154\x70\137\163\151\x74\x65\163")); goto C24ab; Dc2ad: Db632: goto bda9e; acd79: $form->handleRequest($request); goto e38a5; e38a5: if (!(true === $form->isSubmitted())) { goto E9bd7; } goto f4d83; f4d83: $response = $this->handleCreateWordPressSiteForm($request, $form); goto F777d; Ce583: return $response; goto Dc2ad; db15f: if (!(true === $request->isMethod("\120\x4f\x53\x54"))) { goto E4805; } goto acd79; C24ab: goto d953a; goto D27a6; F777d: if (!(false === is_null($response))) { goto Db632; } goto Ce583; E869e: return $response; goto Fffef; d69fb: if (true === $hasPermissions) { goto Ed890; } goto D6a27; bda9e: E9bd7: goto a3aa9; Ff79c: $form = $this->createWordPressSiteForm(); goto db15f; Af4dd: $response = $this->render("\106\x72\157\x6e\164\x65\x6e\x64\57\123\x69\164\x65\57\x4e\145\x77\x2f\167\157\x72\x64\x70\x72\x65\x73\x73\x2e\150\x74\x6d\x6c\x2e\164\167\151\x67", ["\x75\163\x65\x72" => $user, "\146\157\x72\x6d" => $form->createView(), "\146\157\x72\155\105\x72\162\x6f\162\163" => $this->formErrors]); goto C20a3; C20a3: d953a: goto E869e; a3aa9: E4805: goto Af4dd; aa73b: $user = $this->getUser(); goto Ff79c; Fffef: } private function createWordPressSiteForm() : Form { goto acf77; F811d: $form->add("\163\165\142\x6d\x69\164", SubmitType::class, ["\x61\x74\164\162" => ["\x63\x6c\141\163\x73" => "\x62\x74\156\40\x62\164\156\x2d\154\147\x20\x62\164\156\55\x62\x6c\165\145"], "\154\141\142\145\154" => "\x43\x72\x65\x61\x74\x65"]); goto F6c05; F6c05: return $form; goto c00c9; acf77: $form = $this->createForm("\101\x70\x70\134\x46\157\162\155\134\123\x69\164\145\116\x65\x77\x57\157\162\144\x50\162\145\163\x73\124\x79\160\145", [], ["\x61\x63\164\x69\157\156" => $this->generateUrl("\143\154\x70\137\163\151\x74\x65\137\x77\x6f\162\144\160\x72\x65\x73\163\x5f\x6e\145\167"), "\155\x65\x74\150\157\144" => "\120\117\x53\124", "\x61\164\x74\x72" => ["\x69\144" => "\156\x65\x77\x2d\x77\157\162\144\160\162\x65\x73\x73\x2d\163\x69\x74\145\55\x66\157\x72\155"]]); goto F811d; c00c9: } private function handleCreateWordPressSiteForm(Request $request, Form $form) { goto Ab2b6; Ab2b6: if (true === $form->isValid()) { goto e5ca3; } goto a4aeb; f0338: e5ca3: goto E8379; e280a: A4c93: goto D26dc; E8379: try { goto C7a8e; bf408: $adminUserName = $form->get("\141\144\x6d\151\x6e\x55\163\x65\x72\x4e\141\x6d\x65")->getData(); goto Cbb8c; C8c5e: $siteCredentials = [$this->translator->trans("\123\x69\164\x65") => [$this->translator->trans("\x49\120\40\101\x64\144\x72\145\x73\163") => $instance->getIpv4PublicIp(), $this->translator->trans("\104\157\x6d\141\151\156\40\x4e\141\x6d\x65") => sprintf("\x68\x74\164\160\163\x3a\57\57\45\x73", $domainName), $this->translator->trans("\x53\151\x74\145\x20\125\x73\145\x72") => $siteUser, $this->translator->trans("\x50\x61\x73\163\x77\157\x72\x64") => $siteUserPassword]]; goto E34e8; f4bc8: $instance = $request->attributes->get("\x69\156\x73\164\141\156\143\145"); goto C8c5e; Dad88: de52a: goto c172c; Ac999: $databaseName = $this->getUniqueDatabaseName($generatedName); goto be833; B4b6e: $isMultiSite = $form->get("\x6d\x75\x6c\x74\x69\123\x69\164\145")->getData(); goto Ddd9b; Be4a4: $this->installWordPress($isMultiSite, $url, $title, $adminUserName, $adminPassword, $adminEmail, $siteEntity, $activeDatabaseServerEntity, $databaseEntity, $databaseUserEntity); goto e8939; Fd822: $this->siteEntityManager->updateEntity($siteEntity); goto Be4a4; A9218: $generatedName = $this->generateNameFromDomainName($domainName); goto ffa25; Ddd9b: $activeDatabaseServerEntity = $this->databaseServerEntityManager->getActiveDatabaseServer(); goto b46d3; Cc3f4: $databaseManager = new DatabaseManager($activeDatabaseServerEntity); goto Ef1d9; ce229: $varnishCacheSettings = []; goto B8876; c9fc5: $databaseEntity->setName($databaseName); goto D590b; D590b: $databaseEntity->setSite($siteEntity); goto f4bc7; ffa25: $siteUser = $form->get("\x73\151\164\x65\x55\163\x65\162")->getData(); goto Ca187; Cbb8c: $adminPassword = $form->get("\x61\144\155\x69\156\120\141\163\x73\x77\x6f\162\x64")->getData(); goto Df96b; Df96b: $adminEmail = $form->get("\x61\x64\x6d\x69\x6e\x45\155\141\151\154")->getData(); goto B4b6e; C5ceb: $session = $request->getSession(); goto bdd6b; Ca187: $siteUserPassword = $form->get("\163\x69\164\145\x55\163\x65\x72\x50\141\x73\x73\x77\157\x72\144")->getData(); goto af8c3; b46d3: $databaseEntity = $this->databaseEntityManager->createEntity(); goto c9e54; e3ad4: $response = $this->redirect($this->generateUrl("\x63\154\x70\x5f\x73\x69\x74\145\x5f\167\x6f\162\x64\x70\x72\x65\x73\163\x5f\x69\x6e\163\164\141\154\154\145\144")); goto e2279; Ba722: $siteEntity = $this->createPhpSite($domainName, $domainName, $phpVersion, self::WORDPRESS_VHOST_TEMPLATE_NAME, $siteUser, $siteUserPassword, $varnishCacheSettings); goto f6343; d2b79: $url = sprintf("\150\164\x74\160\163\72\x2f\57\45\163", $siteEntity->getDomainName()); goto c0542; b060b: $eventData = ["\x73\151\164\145" => $domainName, "\163\x69\x74\x65\125\163\145\x72" => $siteUser]; goto Ce788; E59c9: $databaseManager->createUser($databaseUserEntity); goto Fd822; C720a: $databaseUserEntity->setPermissions(DatabaseUserEntity::PERMISSIONS_READ_WRITE); goto efc6c; Fdd36: $databaseUserEntity->setUserName($databaseUserName); goto Ae757; c9e54: $databaseEntity->setDatabaseServer($activeDatabaseServerEntity); goto c9fc5; B8876: if (!(false === empty($vhostTemplateEntity->getVarnishCacheSettings()))) { goto Fddfb; } goto A4ff2; C7a8e: $user = $this->getUser(); goto C5ceb; Ef1d9: $databaseManager->createDatabase($databaseEntity); goto E59c9; A4ff2: $varnishCacheSettings = (array) json_decode($vhostTemplateEntity->getVarnishCacheSettings(), true); goto Dbe33; c172c: $session->set("\x73\x69\164\x65\x43\x72\145\144\x65\x6e\x74\x69\x61\154\163", $siteCredentials); goto b060b; f4bc7: $databaseUserEntity = $this->databaseUserEntityManager->createEntity(); goto Fdd36; dca63: $phpVersion = $vhostTemplateEntity->getPhpVersion(); goto ce229; b15ed: $siteCredentials["\127\157\x72\x64\120\162\x65\x73\163"] = [$this->translator->trans("\x41\144\155\151\156\x20\x45\55\x4d\141\151\154") => $adminEmail, $this->translator->trans("\x41\x64\x6d\151\x6e\x20\125\163\145\162\x20\116\x61\x6d\145") => $adminUserName, $this->translator->trans("\x41\x64\155\x69\x6e\x20\x50\141\x73\x73\167\157\162\x64") => $adminPassword, $this->translator->trans("\x41\x64\x6d\x69\x6e\40\125\x72\154") => sprintf("\x68\164\164\160\x73\72\57\x2f\x25\163\57\x77\x70\x2d\141\144\155\x69\x6e\x2f", rtrim($domainName, "\x2f"))]; goto Dad88; f6343: $siteEntity->setApplication($vhostTemplateEntity->getName()); goto f4bc8; e7591: $databaseUserPassword = PasswordGenerator::generate(self::PASSWORD_LENGTH); goto d2b79; efc6c: $databaseUserEntity->setDatabase($databaseEntity); goto B81cb; Ae757: $databaseUserEntity->setPassword(Crypto::encrypt($databaseUserPassword)); goto C720a; E34e8: if (!(false === is_null($siteEntity))) { goto de52a; } goto Ac999; bdd6b: $domainName = $form->get("\144\x6f\x6d\141\151\x6e\x4e\x61\x6d\145")->getData(); goto A9218; c0542: $title = $form->get("\x73\151\x74\x65\x54\x69\164\x6c\145")->getData(); goto bf408; e8939: $siteCredentials[$this->translator->trans("\x44\x61\164\x61\x62\x61\163\145")] = [$this->translator->trans("\x48\157\163\164") => $activeDatabaseServerEntity->getHost(), $this->translator->trans("\x50\x6f\162\x74") => $activeDatabaseServerEntity->getPort(), $this->translator->trans("\104\x61\164\x61\x62\x61\163\145\40\116\141\x6d\x65") => $databaseName, $this->translator->trans("\x44\141\x74\x61\x62\x61\x73\145\x20\125\x73\x65\x72\40\x4e\141\155\145") => $databaseUserName, $this->translator->trans("\x44\141\x74\141\142\141\x73\x65\40\125\163\x65\x72\40\120\141\163\x73\167\157\162\x64") => $databaseUserPassword]; goto b15ed; B81cb: $databaseEntity->addUser($databaseUserEntity); goto bbaf9; Dbe33: Fddfb: goto Ba722; Ce788: EventQueue::addEvent(EventQueue::EVENT_SITE_WORDPRESS_CREATE, $user, $eventData, $request); goto e3ad4; bbaf9: $siteEntity->addDatabase($databaseEntity); goto Cc3f4; e2279: return $response; goto E1118; af8c3: $vhostTemplateEntity = $this->getVhostTemplateEntity(self::WORDPRESS_VHOST_TEMPLATE_NAME); goto dca63; be833: $databaseUserName = $this->getUniqueDatabaseUserName($generatedName); goto e7591; E1118: } catch (\Exception $e) { $this->logger->exception($e); $session->getFlashBag()->set("\x64\141\x6e\147\x65\x72", $this->translator->trans("\x41\x6e\40\x65\162\x72\157\x72\40\150\141\x73\40\x6f\143\x63\x75\162\162\145\144\54\40\145\x72\x72\157\162\x20\x6d\x65\x73\163\141\147\x65\72\40\x25\x65\162\x72\x6f\162\115\x65\163\163\x61\147\x65\x25", ["\45\145\162\162\157\x72\115\145\x73\163\x61\147\x65\x25" => $e->getMessage()])); } goto e280a; a4aeb: $this->formErrors = $this->getErrorMessages($form); goto Cdbe2; Cdbe2: goto A4c93; goto f0338; D26dc: } private function installWordPress(bool $isMultiSite, string $url, string $title, string $adminUserName, string $adminPassword, string $adminEmail, SiteEntity $siteEntity, DatabaseServerEntity $databaseServerEntity, DatabaseEntity $databaseEntity, DatabaseUserEntity $databaseUserEntity) : void { goto B4e37; b57bd: $wordPressInstaller->installCore($isMultiSite, $url, $title, $adminUserName, $adminPassword, $adminEmail); goto da013; eeb5d: $wordPressInstaller->resetPermissions(); goto Ceda3; e83a5: $databaseName = $databaseEntity->getName(); goto E6402; c2076: $wordPressInstaller = new WordPressInstaller($siteEntity); goto Bac73; b483e: E51bb: goto F2222; f6cb6: Cdae8: goto eeb5d; ca577: e679e: goto f6cb6; E082c: foreach ($this->wordPressMultiSiteConfigValues as $key => $config) { $wordPressInstaller->setConfigValue($key, $config["\x76\141\154\165\x65"], $config["\162\x61\167"]); c7afa: } goto ca577; e2fce: $wordPressInstaller->createConfig($databaseHost, $databaseName, $databaseUserName, $databaseUserPassword); goto b57bd; a4810: $databaseUserPassword = Crypto::decrypt($databaseUserEntity->getPassword()); goto c2076; da013: foreach ($this->wordPressDefaultConfigValues as $key => $config) { $wordPressInstaller->setConfigValue($key, $config["\166\141\x6c\165\145"], $config["\x72\141\x77"]); E16c4: } goto b483e; E8fb4: $this->wordPressMultiSiteConfigValues["\104\117\x4d\101\111\x4e\x5f\103\125\122\122\105\x4e\124\137\123\111\124\x45"]["\x76\x61\154\165\145"] = $siteEntity->getDomainName(); goto E082c; B4e37: $databaseHost = sprintf("\x25\x73\x3a\x25\x73", $databaseServerEntity->getHost(), $databaseServerEntity->getPort()); goto e83a5; E6402: $databaseUserName = $databaseUserEntity->getUserName(); goto a4810; F2222: if (!(true === $isMultiSite)) { goto Cdae8; } goto E8fb4; Bac73: $wordPressInstaller->downloadAndExtractLatestVersion(); goto e2fce; Ceda3: } private function createPhpSite(string $domainName, string $rootDirectory, string $phpVersion, string $vhostTemplateName, string $siteUser, string $siteUserPassword, array $varnishCacheSettings = []) : SiteEntity { goto B125b; a5857: $phpSiteCreator->createRootDirectory(); goto a08a8; Bcec6: $phpSite->setRootDirectory($siteEntity->getRootDirectory()); goto C4ef1; Ac538: $errorMessages = $this->getConstraintViolationListErrorMessages($constraintViolationList); goto e55fe; c0a84: $phpSettingsEntity = new PhpSettingsEntity(); goto E5837; A7433: $siteEntity = $this->siteEntityManager->createEntity(); goto d0e93; af713: $phpSite->setSubdomain($subdomain); goto Bcec6; a08a8: $phpSiteCreator->createLogrotateFile(); goto E3ccd; D273b: $redirectionVhostTemplateFile = realpath(dirname(__FILE__) . "\57\x2e\x2e\x2f\56\x2e\57\56\56\57\x72\145\x73\x6f\x75\162\x63\x65\163\x2f\x6e\147\151\156\170\57\166\x68\157\x73\x74\137\164\x65\x6d\x70\x6c\x61\x74\145\57\162\x65\x64\x69\x72\145\x63\x74\55\x68\164\164\x70\63"); goto b2623; ba11b: $certificateEntity->setType(CertificateEntity::TYPE_SELF_SIGNED); goto a04fb; f865d: $siteConstraints = $this->validator->validate($siteEntity); goto cc59b; E3ccd: $phpSiteCreator->createIndexPhp(); goto Da685; F290a: $phpSite->setPhpSettings($phpSettingsEntity); goto Edd90; C4ef1: $phpSite->setCertificate($certificateEntity); goto F290a; f9a27: goto Da08d; goto e99f6; f3686: if (!(false === is_null($subdomain) && "\x77\167\x77" == $subdomain)) { goto d5833; } goto ae2c1; E56fc: a83b8: goto f3686; D915b: $vhostTemplate->build(); goto a614f; ac737: if (!(count($siteConstraints) || count($phpSettingsConstraints))) { goto Eaaa8; } goto Bdf0a; Decbd: b52f8: goto D118e; C3606: $selfSignedCertificate = Openssl::createSelfSignedCertificate($privateKey, $csr); goto dda34; a0ba5: if (!(false === empty($varnishCache))) { goto d3702; } goto e91a2; A03be: $csr = $csrGenerator->generate(); goto C3606; C31ef: $phpSettingsEntity->setSite($siteEntity); goto f865d; d0e93: $siteEntity->setType(SiteEntity::TYPE_PHP); goto F6ea9; E8ffa: $subdomain = $resolvedDomainName->subDomain()->toString(); goto c56d8; E03c7: $redirectionVhostTemplateFile = realpath(dirname(__FILE__) . "\x2f\x2e\x2e\57\56\56\57\56\56\x2f\162\145\x73\157\x75\162\143\145\163\x2f\156\147\x69\156\170\57\166\x68\157\x73\164\137\x74\145\x6d\160\x6c\x61\164\145\x2f\x72\x65\x64\151\162\x65\143\x74"); goto f9a27; e77b0: $vhostTemplateEntity = $this->getVhostTemplateEntity($vhostTemplateName); goto be536; D2659: $certificateEntity = $this->certificateEntityManager->createEntity(); goto B8f48; E92ab: $vhostTemplate = sprintf("\45\x73\x25\163", $redirectionVhostTemplate, $vhostTemplate); goto Ddf8f; Ddf8f: A2641: goto fa40f; cc59b: $phpSettingsConstraints = $this->validator->validate($phpSettingsEntity); goto ac737; e8bb8: $phpSite->setUserPassword($siteEntity->getUserPassword()); goto B7c3e; Ee2f6: d3702: goto e77e3; a9823: $subjectAlternativeNames[] = sprintf("\x77\x77\x77\56\45\163", $domainName); goto E56fc; e91a2: $siteEntity->setVarnishCache(true); goto Ee2f6; B125b: $varnishCache = false === empty($varnishCacheSettings) ? true : false; goto Ba1ad; ba354: Eaaa8: goto D2659; ec54d: $vhostTemplate = new PhpVhostTemplate($phpSite); goto E671c; Ad815: $phpSite->setUser($siteEntity->getUser()); goto e8bb8; e55fe: throw new \Exception(implode("\73\x20", $errorMessages)); goto ba354; dda34: $certificateEntity->setDefaultCertificate(true); goto ba11b; fedf8: $registrableDomain = $resolvedDomainName->registrableDomain()->toString(); goto E8ffa; B743a: d5833: goto a5522; a5522: $distinguishedName = new DistinguishedName($domainName, $subjectAlternativeNames); goto D14ce; C870f: return $siteEntity; goto a5ff8; a4ca4: $subjectAlternativeNames = []; goto e127e; Fa420: $phpSite = new PhpSite(); goto Ad815; Ba1ad: $rootDirectory = trim(rtrim(ltrim($rootDirectory, "\x2f")), "\x2f"); goto Cb1dd; ae2c1: $subjectAlternativeNames[] = $registrableDomain; goto B743a; A9df1: $privateKey = $rsaKeyGenerator->generatePrivateKey(); goto a4ca4; e127e: if (!(true === is_null($subdomain))) { goto a83b8; } goto a9823; B7c3e: $phpSite->setDomainName($domainName); goto f9b04; B8f48: $certificateEntity->setSite($siteEntity); goto Cc98d; F8785: $phpSiteCreator->resetPermissions(); goto ec54d; e9259: $vhostTemplate->addProcessor(new RedirectDomainProcessor()); goto D915b; D421c: $phpSite->setVarnishCacheSettings($varnishCacheSettings); goto Decbd; a04fb: $certificateEntity->setCsr($csr); goto ec345; Cb1dd: $resolvedDomainName = $this->domainNameParser->resolveDomainName($domainName); goto fedf8; Af7ab: $siteEntity->setUser($siteUser); goto Cb13c; a614f: $siteEntity->setVhostTemplate($vhostTemplate->getContent()); goto fa573; e77e3: if (!(true === is_null($subdomain) || "\x77\167\x77" == $subdomain)) { goto C2054; } goto e7a02; ec345: $certificateEntity->setPrivateKey($privateKey->getPEM()); goto B5d73; F4cf8: if (!(false === empty($redirectionVhostTemplate))) { goto A2641; } goto E92ab; F6ea9: $siteEntity->setDomainName($domainName); goto Af7ab; e8dfc: $phpSiteCreator->reloadNginxService(); goto F8785; e4ddb: $vhostTemplate->addProcessor(new ServerNameProcessor()); goto eefbe; e7a02: if (true === isset($_ENV["\101\120\x50\137\x48\124\124\120\x33"]) && "\164\162\165\145" === $_ENV["\x41\x50\120\x5f\x48\124\x54\120\x33"]) { goto C3b44; } goto E03c7; c56d8: $subdomain = false === empty($subdomain) ? $subdomain : null; goto e77b0; D715a: $constraintViolationList->addAll($siteConstraints); goto e973c; B5fd7: $defaultVarnishCacheSettings = ["\x65\x6e\141\142\x6c\x65\x64" => false, "\x73\x65\162\x76\x65\162" => self::VARNISH_CACHE_SERVER, "\x63\x61\x63\x68\145\x54\141\147\x50\x72\x65\x66\151\x78" => substr(md5(time()), 0, 4)]; goto B41f1; Edd90: $phpSite->setVhostTemplate($siteEntity->getVhostTemplate()); goto Add13; ee0c8: if (!(true === $varnishCache)) { goto b52f8; } goto Ba8db; Da685: $phpSiteCreator->createPrivateKeyAndCertificate(); goto Cd24f; e973c: $constraintViolationList->addAll($phpSettingsConstraints); goto Ac538; Add13: $phpSiteCreator = new PhpSiteCreator($phpSite); goto f4ff6; fc930: $siteEntity->setCertificate($certificateEntity); goto Fa420; fa573: $siteEntity->setApplication($vhostTemplateEntity->getName()); goto be99a; be99a: $this->siteEntityManager->updateEntity($siteEntity); goto C870f; b2623: Da08d: goto d2a73; D14ce: $csrGenerator = new CsrGenerator($privateKey, $distinguishedName); goto A03be; d2a73: $redirectionVhostTemplate = file_get_contents($redirectionVhostTemplateFile); goto F4cf8; B5d73: $certificateEntity->setCertificate($selfSignedCertificate); goto fc930; Cb13c: $siteEntity->setUserPassword($siteUserPassword); goto f88be; E5837: $siteEntity->setPhpSettings($phpSettingsEntity); goto C5ff6; f4ff6: $phpSiteCreator->createUser(); goto a5857; Cd24f: $phpSiteCreator->createPhpFpmPool(); goto e78f4; Bdf0a: $constraintViolationList = new ConstraintViolationList(); goto D715a; be536: $vhostTemplate = $vhostTemplateEntity->getTemplate(); goto A7433; C5ff6: $phpSettingsEntity->setPhpVersion($phpVersion); goto C31ef; B41f1: $varnishCacheSettings = array_merge($defaultVarnishCacheSettings, $varnishCacheSettings); goto c8b4a; fa40f: C2054: goto de96d; e78f4: $phpSiteCreator->reloadPhpFpmService(); goto ee0c8; eefbe: $vhostTemplate->addProcessor(new RedirectServerNameProcessor()); goto e9259; D118e: $phpSiteCreator->createNginxVhost(); goto e8dfc; Cc98d: $rsaKeyGenerator = new RsaKeyGenerator(); goto A9df1; Ba8db: $phpSite->setVarnishCache(true); goto B5fd7; ab264: $vhostTemplate->resetProcessors(); goto e4ddb; f9b04: $phpSite->setRegistrableDomain($registrableDomain); goto af713; e99f6: C3b44: goto D273b; de96d: $siteEntity->setVhostTemplate($vhostTemplate); goto c0a84; E671c: $vhostTemplate->setContent($siteEntity->getVhostTemplate()); goto ab264; f88be: $siteEntity->setRootDirectory($rootDirectory); goto a0ba5; c8b4a: $phpSiteCreator->createVarnishCacheStructure($varnishCacheSettings); goto D421c; a5ff8: } private function createNodejsSite(string $domainName, string $nodejsVersion, string $port, string $siteUser, string $siteUserPassword) : SiteEntity { goto eec03; ae631: C7d0f: goto a1ac2; d600f: $vhostTemplate->addProcessor(new ServerNameProcessor()); goto da993; f9799: $siteEntity->setVhostTemplate($vhostTemplate->getContent()); goto Ab1f9; a971b: $nodejsSite->setNodejsSettings($nodejsSettingsEntity); goto Db69e; d1043: $subjectAlternativeNames = []; goto D7882; Cba84: $nodejsSite->setUser($siteEntity->getUser()); goto C079d; Fad3a: $siteEntity->setVhostTemplate($vhostTemplate); goto c8ac5; ae7d6: $subjectAlternativeNames[] = sprintf("\167\x77\x77\x2e\45\163", $domainName); goto f1662; F02df: Ad255: goto e13d0; acd0e: $siteEntity->setUser($siteUser); goto cefa0; afa18: $rsaKeyGenerator = new RsaKeyGenerator(); goto f7c3b; b50a1: $nodejsSiteCreator->resetPermissions(); goto b5c56; F2667: $vhostTemplate->resetProcessors(); goto d600f; C6966: $constraintViolationList->addAll($nodejsSettingsConstraints); goto fbea7; E035e: $nodejsSite->setDomainName($domainName); goto ca207; F44d3: $vhostTemplate->build(); goto f9799; ca207: $nodejsSite->setRegistrableDomain($registrableDomain); goto e260c; Dca20: goto Ad255; goto Ab526; f8b9e: $certificateEntity->setDefaultCertificate(true); goto dd77d; A4a5d: $certificateEntity = $this->certificateEntityManager->createEntity(); goto Aab5f; a2d5c: if (true === isset($_ENV["\101\120\120\137\110\x54\x54\120\x33"]) && "\164\x72\x75\x65" === $_ENV["\x41\x50\x50\x5f\110\x54\x54\x50\x33"]) { goto Fa03b; } goto da9a3; a1ac2: $distinguishedName = new DistinguishedName($domainName, $subjectAlternativeNames); goto cc660; fbea7: $errorMessages = $this->getConstraintViolationListErrorMessages($constraintViolationList); goto C42e7; Aab5f: $certificateEntity->setSite($siteEntity); goto afa18; Ab526: Fa03b: goto fc050; e9d94: $nodejsSiteCreator->createRootDirectory(); goto C954c; cc8b8: $this->siteEntityManager->updateEntity($siteEntity); goto C077d; f1aa5: $nodejsSiteCreator->createNginxVhost(); goto ca214; efec5: $csr = $csrGenerator->generate(); goto D6481; d484e: $nodejsSettingsEntity->setPort($port); goto Eb997; cefa0: $siteEntity->setUserPassword($siteUserPassword); goto E8df6; Beabc: $subdomain = $resolvedDomainName->subDomain()->toString(); goto d046f; Beba0: $vhostTemplateEntity = $this->getVhostTemplateEntity(self::NODEJS_VHOST_TEMPLATE_NAME); goto C0002; C954c: $nodejsSiteCreator->createLogrotateFile(); goto D457f; C0002: $vhostTemplate = $vhostTemplateEntity->getTemplate(); goto c1bae; F445f: if (!(true === is_null($subdomain) || "\x77\x77\167" == $subdomain)) { goto d9332; } goto a2d5c; Cdd1e: $nodejsSettingsEntity->setNodejsVersion($nodejsVersion); goto d484e; f7c3b: $privateKey = $rsaKeyGenerator->generatePrivateKey(); goto d1043; c8ac5: $nodejsSettingsEntity = new NodejsSettingsEntity(); goto D9c41; c8007: $nodejsSiteCreator->createPrivateKeyAndCertificate(); goto f1aa5; E5d66: $nodejsSiteCreator->createUser(); goto e9d94; dd77d: $certificateEntity->setType(CertificateEntity::TYPE_SELF_SIGNED); goto B5fff; cc660: $csrGenerator = new CsrGenerator($privateKey, $distinguishedName); goto efec5; Bec7c: $nodejsSettingsConstraints = $this->validator->validate($nodejsSettingsEntity); goto B9c01; ca214: $nodejsSiteCreator->reloadNginxService(); goto b50a1; da993: $vhostTemplate->addProcessor(new RedirectServerNameProcessor()); goto b80bf; B365b: $nodejsSite->setCertificate($certificateEntity); goto a971b; Be521: $siteConstraints = $this->validator->validate($siteEntity); goto Bec7c; ad1f6: $constraintViolationList = new ConstraintViolationList(); goto E485b; caa8a: $nodejsSiteCreator = new NodejsSiteCreator($nodejsSite); goto E5d66; B9c01: if (!(count($siteConstraints) || count($nodejsSettingsConstraints))) { goto B1897; } goto ad1f6; F482e: $siteEntity->setDomainName($domainName); goto acd0e; A1d9a: $vhostTemplate->setContent($siteEntity->getVhostTemplate()); goto F2667; e13d0: $redirectionVhostTemplate = file_get_contents($redirectionVhostTemplateFile); goto d5343; B2860: $subjectAlternativeNames[] = $registrableDomain; goto ae631; E485b: $constraintViolationList->addAll($siteConstraints); goto C6966; F2f36: $siteEntity->setCertificate($certificateEntity); goto ba0e6; eec03: $resolvedDomainName = $this->domainNameParser->resolveDomainName($domainName); goto c760d; Dc89b: cadf5: goto D71ac; Ab1f9: $siteEntity->setApplication($vhostTemplateEntity->getName()); goto cc8b8; c760d: $registrableDomain = $resolvedDomainName->registrableDomain()->toString(); goto Beabc; fc050: $redirectionVhostTemplateFile = realpath(dirname(__FILE__) . "\57\56\56\x2f\56\x2e\x2f\x2e\x2e\x2f\x72\145\x73\x6f\x75\x72\x63\145\x73\x2f\x6e\x67\x69\156\170\57\166\150\157\x73\164\x5f\164\145\155\160\x6c\x61\x74\145\x2f\x72\x65\x64\x69\162\x65\x63\x74\55\x68\164\x74\160\63"); goto F02df; ba0e6: $nodejsSite = new NodejsSite(); goto Cba84; D71ac: d9332: goto Fad3a; D6481: $selfSignedCertificate = Openssl::createSelfSignedCertificate($privateKey, $csr); goto f8b9e; C7366: B1897: goto A4a5d; b5c56: $vhostTemplate = new StaticVhostTemplate($nodejsSite); goto A1d9a; b80bf: $vhostTemplate->addProcessor(new RedirectDomainProcessor()); goto F44d3; B5fff: $certificateEntity->setCsr($csr); goto ff887; ff887: $certificateEntity->setPrivateKey($privateKey->getPEM()); goto c8e65; E8df6: $siteEntity->setRootDirectory($domainName); goto F445f; Eb997: $nodejsSettingsEntity->setSite($siteEntity); goto Be521; da9a3: $redirectionVhostTemplateFile = realpath(dirname(__FILE__) . "\x2f\56\x2e\x2f\x2e\x2e\x2f\56\x2e\57\x72\x65\x73\157\165\162\x63\145\163\x2f\156\147\x69\x6e\x78\x2f\166\x68\x6f\163\x74\x5f\x74\145\x6d\160\154\141\164\145\x2f\x72\x65\144\x69\162\145\x63\x74"); goto Dca20; a7e51: $vhostTemplate = sprintf("\x25\x73\x25\x73", $redirectionVhostTemplate, $vhostTemplate); goto Dc89b; C42e7: throw new \Exception(implode("\x3b\40", $errorMessages)); goto C7366; C079d: $nodejsSite->setUserPassword($siteEntity->getUserPassword()); goto E035e; f1662: E0701: goto C445d; e260c: $nodejsSite->setSubdomain($subdomain); goto D4cf7; c8e65: $certificateEntity->setCertificate($selfSignedCertificate); goto F2f36; D4cf7: $nodejsSite->setRootDirectory($siteEntity->getRootDirectory()); goto B365b; D457f: $nodejsSiteCreator->createNvmDirectory(); goto d8aee; d046f: $subdomain = false === empty($subdomain) ? $subdomain : null; goto Beba0; d8aee: $nodejsSiteCreator->installNodejs(); goto c8007; Db69e: $nodejsSite->setVhostTemplate($siteEntity->getVhostTemplate()); goto caa8a; Bac5b: $siteEntity->setType(SiteEntity::TYPE_NODEJS); goto F482e; D9c41: $siteEntity->setNodejsSettings($nodejsSettingsEntity); goto Cdd1e; d5343: if (!(false === empty($redirectionVhostTemplate))) { goto cadf5; } goto a7e51; C445d: if (!(false === is_null($subdomain) && "\167\167\x77" == $subdomain)) { goto C7d0f; } goto B2860; C077d: return $siteEntity; goto fc5e7; c1bae: $siteEntity = $this->siteEntityManager->createEntity(); goto Bac5b; D7882: if (!(true === is_null($subdomain))) { goto E0701; } goto ae7d6; fc5e7: } private function createPythonSite(string $domainName, string $pythonVersion, string $port, string $siteUser, string $siteUserPassword) : SiteEntity { goto cd95b; f561e: if (!(count($siteConstraints) || count($pythonSettingsConstraints))) { goto c9fb6; } goto a945b; Ca7c9: $certificateEntity->setType(CertificateEntity::TYPE_SELF_SIGNED); goto C0c04; bfedf: $pythonSite->setPythonSettings($pythonSettingsEntity); goto d22bc; A7518: $pythonSiteCreator->reloadNginxService(); goto cf7c1; Ddbe1: $redirectionVhostTemplateFile = realpath(dirname(__FILE__) . "\x2f\56\x2e\x2f\56\x2e\x2f\x2e\x2e\57\162\145\x73\x6f\x75\162\x63\145\163\x2f\156\147\151\156\x78\x2f\166\x68\x6f\x73\x74\137\164\145\155\160\x6c\x61\164\145\x2f\x72\145\x64\151\x72\x65\143\164"); goto A40a9; A0236: $registrableDomain = $resolvedDomainName->registrableDomain()->toString(); goto f5b92; C4366: d60a0: goto cd130; D8b5f: $siteEntity->setDomainName($domainName); goto Acb73; e01f1: $certificateEntity->setSite($siteEntity); goto f6a1d; C1d4c: $vhostTemplateEntity = $this->getVhostTemplateEntity(self::PYTHON_VHOST_TEMPLATE_NAME); goto B6a44; B042a: $pythonSiteCreator->createNginxVhost(); goto A7518; D2335: $certificateEntity->setDefaultCertificate(true); goto Ca7c9; ca9d3: $errorMessages = $this->getConstraintViolationListErrorMessages($constraintViolationList); goto E11ef; b4fa0: $pythonSiteCreator->createRootDirectory(); goto bef55; a1b33: $vhostTemplate = new PythonVhostTemplate($pythonSite); goto c0614; B6b76: $siteEntity->setVhostTemplate($vhostTemplate->getContent()); goto d3835; C0c04: $certificateEntity->setCsr($csr); goto c0538; B6475: eae3f: goto Ea7c3; bbad2: $pythonSiteCreator->writePythonVersionFile(); goto Ad3a1; dee25: $pythonSite->setUser($siteEntity->getUser()); goto f2baf; A0a47: $vhostTemplate = sprintf("\45\x73\x25\x73", $redirectionVhostTemplate, $vhostTemplate); goto B6475; Fa4eb: $pythonSettingsEntity->setPythonVersion($pythonVersion); goto Fcfd9; dee2f: $siteEntity->setCertificate($certificateEntity); goto d5599; a2fbf: $pythonSite->setCertificate($certificateEntity); goto bfedf; Bd5b5: $csr = $csrGenerator->generate(); goto F7c0d; b77fe: $pythonSiteCreator = new PythonSiteCreator($pythonSite); goto d83df; f5651: $subjectAlternativeNames[] = sprintf("\x77\x77\167\x2e\x25\x73", $domainName); goto aa840; d5599: $pythonSite = new PythonSite(); goto dee25; fde0b: $constraintViolationList->addAll($pythonSettingsConstraints); goto ca9d3; f5b92: $subdomain = $resolvedDomainName->subDomain()->toString(); goto Dacbf; Ad3a1: $pythonSiteCreator->createPrivateKeyAndCertificate(); goto B042a; d83df: $pythonSiteCreator->createUser(); goto b4fa0; c0614: $vhostTemplate->setContent($siteEntity->getVhostTemplate()); goto a67a5; e2620: $pythonSettingsEntity->setSite($siteEntity); goto aacf2; aacf2: $siteConstraints = $this->validator->validate($siteEntity); goto B6f42; aa840: aa4ba: goto a57f3; Ea7c3: Ac12e: goto C93f4; Aac52: $vhostTemplate->addProcessor(new ServerNameProcessor()); goto E2c7e; e9c52: $pythonSite->setRootDirectory($siteEntity->getRootDirectory()); goto a2fbf; cfc07: $siteEntity = $this->siteEntityManager->createEntity(); goto D24d1; c1f39: if (true === isset($_ENV["\x41\120\120\x5f\x48\124\124\120\x33"]) && "\x74\162\x75\145" === $_ENV["\x41\120\120\137\x48\124\x54\120\x33"]) { goto cbe20; } goto Ddbe1; B6f42: $pythonSettingsConstraints = $this->validator->validate($pythonSettingsEntity); goto f561e; B6a44: $vhostTemplate = $vhostTemplateEntity->getTemplate(); goto cfc07; c2a6b: $siteEntity->setRootDirectory($domainName); goto deec2; Fcfd9: $pythonSettingsEntity->setPort($port); goto e2620; D6239: Ebcbd: goto Bf801; c0538: $certificateEntity->setPrivateKey($privateKey->getPEM()); goto fa49e; c4ebe: $subjectAlternativeNames = []; goto e6643; cd130: $redirectionVhostTemplate = file_get_contents($redirectionVhostTemplateFile); goto D7c79; e6643: if (!(true === is_null($subdomain))) { goto aa4ba; } goto f5651; f5824: $pythonSite->setRegistrableDomain($registrableDomain); goto D8103; E11ef: throw new \Exception(implode("\x3b\40", $errorMessages)); goto b24c9; a945b: $constraintViolationList = new ConstraintViolationList(); goto Dd3eb; Dacbf: $subdomain = false === empty($subdomain) ? $subdomain : null; goto C1d4c; f6a1d: $rsaKeyGenerator = new RsaKeyGenerator(); goto Bfcbb; cd95b: $resolvedDomainName = $this->domainNameParser->resolveDomainName($domainName); goto A0236; E2c7e: $vhostTemplate->addProcessor(new RedirectServerNameProcessor()); goto f0bb0; D8103: $pythonSite->setSubdomain($subdomain); goto e9c52; Bfcbb: $privateKey = $rsaKeyGenerator->generatePrivateKey(); goto c4ebe; F7c0d: $selfSignedCertificate = Openssl::createSelfSignedCertificate($privateKey, $csr); goto D2335; Cabf2: $vhostTemplate->build(); goto B6b76; C93f4: $siteEntity->setVhostTemplate($vhostTemplate); goto f998c; Acb73: $siteEntity->setUser($siteUser); goto D5eb8; cf7c1: $pythonSiteCreator->resetPermissions(); goto a1b33; A40a9: goto d60a0; goto C6cc0; C6cc0: cbe20: goto dd6f5; D5eb8: $siteEntity->setUserPassword($siteUserPassword); goto c2a6b; fa49e: $certificateEntity->setCertificate($selfSignedCertificate); goto dee2f; Bbbf8: $subjectAlternativeNames[] = $registrableDomain; goto D6239; b215c: $siteEntity->setApplication($vhostTemplateEntity->getName()); goto db7b8; deec2: if (!(true === is_null($subdomain) || "\x77\x77\x77" == $subdomain)) { goto Ac12e; } goto c1f39; f0bb0: $vhostTemplate->addProcessor(new RedirectDomainProcessor()); goto Cabf2; e8e62: $csrGenerator = new CsrGenerator($privateKey, $distinguishedName); goto Bd5b5; f2baf: $pythonSite->setUserPassword($siteEntity->getUserPassword()); goto C5d80; dd6f5: $redirectionVhostTemplateFile = realpath(dirname(__FILE__) . "\57\56\56\57\x2e\56\57\56\x2e\x2f\162\145\x73\157\165\x72\143\145\163\x2f\156\147\x69\x6e\x78\57\166\x68\x6f\x73\x74\137\164\145\x6d\x70\154\x61\x74\x65\57\x72\x65\144\151\162\145\143\164\55\150\x74\x74\x70\x33"); goto C4366; ebe8a: $siteEntity->setPythonSettings($pythonSettingsEntity); goto Fa4eb; C5d80: $pythonSite->setDomainName($domainName); goto f5824; bef55: $pythonSiteCreator->createLogrotateFile(); goto bbad2; D24d1: $siteEntity->setType(SiteEntity::TYPE_PYTHON); goto D8b5f; a67a5: $vhostTemplate->resetProcessors(); goto Aac52; db7b8: return $siteEntity; goto bb293; f998c: $pythonSettingsEntity = new PythonSettingsEntity(); goto ebe8a; b24c9: c9fb6: goto C2bd7; d3835: $this->siteEntityManager->updateEntity($siteEntity); goto b215c; Dd3eb: $constraintViolationList->addAll($siteConstraints); goto fde0b; Bf801: $distinguishedName = new DistinguishedName($domainName, $subjectAlternativeNames); goto e8e62; d22bc: $pythonSite->setVhostTemplate($siteEntity->getVhostTemplate()); goto b77fe; a57f3: if (!(false === is_null($subdomain) && "\x77\167\167" == $subdomain)) { goto Ebcbd; } goto Bbbf8; C2bd7: $certificateEntity = $this->certificateEntityManager->createEntity(); goto e01f1; D7c79: if (!(false === empty($redirectionVhostTemplate))) { goto eae3f; } goto A0a47; bb293: } private function createStaticSite(string $domainName, string $siteUser, string $siteUserPassword) : SiteEntity { goto B0a72; d88a4: $staticSiteCreator->createIndexHtml(); goto eda97; db2e4: $constraintViolationList->addAll($siteConstraints); goto Bff54; e7b6f: $redirectionVhostTemplateFile = realpath(dirname(__FILE__) . "\x2f\x2e\56\x2f\x2e\56\57\x2e\x2e\x2f\162\145\163\x6f\x75\x72\143\145\x73\x2f\x6e\147\151\x6e\170\x2f\166\x68\157\x73\164\x5f\x74\145\155\x70\154\141\164\145\x2f\162\x65\x64\x69\162\x65\x63\x74"); goto Eb34c; A4416: Bfc37: goto C7654; A8fef: $staticSiteCreator->createLogrotateFile(); goto d88a4; a9b9a: $vhostTemplate = new StaticVhostTemplate($staticSite); goto e45fe; B8c29: $staticSiteCreator->reloadNginxService(); goto Fe072; C5159: $certificateEntity->setPrivateKey($privateKey->getPEM()); goto A49f2; e4dd3: $vhostTemplate->build(); goto ee759; a57c9: $staticSiteCreator->createRootDirectory(); goto A8fef; b245e: $vhostTemplate = $vhostTemplateEntity->getTemplate(); goto Cf982; B47a1: return $siteEntity; goto db8d0; caab9: de072: goto Daa22; Ddb1a: c63ae: goto f6f5b; f1e30: $staticSiteCreator->createNginxVhost(); goto B8c29; F8335: $siteEntity->setUserPassword($siteUserPassword); goto ec768; F530d: $staticSite->setDomainName($domainName); goto c633c; D8967: $subdomain = false === empty($subdomain) ? $subdomain : null; goto Ddb81; F9295: $csr = $csrGenerator->generate(); goto Eefa8; c633c: $staticSite->setRegistrableDomain($registrableDomain); goto fa14b; aeb30: $staticSite = new StaticSite(); goto D262c; fa09f: $certificateEntity->setType(CertificateEntity::TYPE_SELF_SIGNED); goto b6915; Bff54: $errorMessages = $this->getConstraintViolationListErrorMessages($constraintViolationList); goto cb579; Daa22: $certificateEntity = $this->certificateEntityManager->createEntity(); goto B1cbe; A1e1f: $staticSiteCreator->createUser(); goto a57c9; Ddb81: $vhostTemplateEntity = $this->getVhostTemplateEntity(self::STATIC_SITE_VHOST_TEMPLATE_NAME); goto b245e; Fff37: $constraintViolationList = new ConstraintViolationList(); goto db2e4; Fe072: $staticSiteCreator->resetPermissions(); goto a9b9a; ec768: $siteEntity->setRootDirectory($domainName); goto B5315; Eefa8: $selfSignedCertificate = Openssl::createSelfSignedCertificate($privateKey, $csr); goto D0201; d0bfe: $vhostTemplate = sprintf("\x25\x73\x25\x73", $redirectionVhostTemplate, $vhostTemplate); goto e01bc; a59a9: if (true === isset($_ENV["\101\x50\x50\137\110\124\x54\120\x33"]) && "\164\162\165\x65" === $_ENV["\101\120\x50\137\110\124\x54\120\x33"]) { goto c63ae; } goto e7b6f; d65c7: a18cc: goto Ab426; d12c9: if (!count($siteConstraints)) { goto de072; } goto Fff37; eda97: $staticSiteCreator->createPrivateKeyAndCertificate(); goto f1e30; Cf982: $siteEntity = $this->siteEntityManager->createEntity(); goto Dc0ce; da0b1: $vhostTemplate->addProcessor(new RedirectServerNameProcessor()); goto eeecd; A8551: $vhostTemplate->addProcessor(new ServerNameProcessor()); goto da0b1; e8ca8: $siteEntity->setUser($siteUser); goto F8335; A49f2: $certificateEntity->setCertificate($selfSignedCertificate); goto A5fc0; Cf3ca: if (!(true === is_null($subdomain))) { goto A004f; } goto D6603; f6f5b: $redirectionVhostTemplateFile = realpath(dirname(__FILE__) . "\57\56\x2e\57\x2e\56\57\56\x2e\57\162\x65\x73\157\165\x72\x63\x65\x73\57\x6e\147\151\x6e\170\x2f\x76\x68\x6f\163\164\137\x74\x65\155\160\x6c\141\164\145\x2f\x72\x65\x64\151\162\145\x63\x74\x2d\150\x74\164\160\63"); goto d65c7; bbbd9: $subjectAlternativeNames = []; goto Cf3ca; B1cbe: $certificateEntity->setSite($siteEntity); goto F3dce; E4920: $staticSiteCreator = new StaticSiteCreator($staticSite); goto A1e1f; Cdac4: $staticSite->setVhostTemplate($siteEntity->getVhostTemplate()); goto E4920; F627b: $staticSite->setUserPassword($siteEntity->getUserPassword()); goto F530d; fa14b: $staticSite->setSubdomain($subdomain); goto da07c; A8cf4: $siteEntity->setDomainName($domainName); goto e8ca8; Fb39b: if (!(false === is_null($subdomain) && "\x77\167\167" == $subdomain)) { goto Bfc37; } goto b66d2; Eab51: $siteEntity->setApplication($vhostTemplateEntity->getName()); goto a1f14; eeecd: $vhostTemplate->addProcessor(new RedirectDomainProcessor()); goto e4dd3; E5538: $siteEntity->setVhostTemplate($vhostTemplate); goto f7fab; da07c: $staticSite->setRootDirectory($siteEntity->getRootDirectory()); goto A124c; a1f14: $this->siteEntityManager->updateEntity($siteEntity); goto B47a1; A124c: $staticSite->setCertificate($certificateEntity); goto Cdac4; fe9e0: if (!(false === empty($redirectionVhostTemplate))) { goto E4dac; } goto d0bfe; e01bc: E4dac: goto ce50c; ee4cb: A004f: goto Fb39b; b66d2: $subjectAlternativeNames[] = $registrableDomain; goto A4416; Ce8b2: $subdomain = $resolvedDomainName->subDomain()->toString(); goto D8967; e1b7a: $registrableDomain = $resolvedDomainName->registrableDomain()->toString(); goto Ce8b2; B5315: if (!(true === is_null($subdomain) || "\x77\167\167" == $subdomain)) { goto Be891; } goto a59a9; ce50c: Be891: goto E5538; f7fab: $siteConstraints = $this->validator->validate($siteEntity); goto d12c9; b6915: $certificateEntity->setCsr($csr); goto C5159; D262c: $staticSite->setUser($siteEntity->getUser()); goto F627b; b1bc8: $vhostTemplate->resetProcessors(); goto A8551; cb579: throw new \Exception(implode("\x3b\40", $errorMessages)); goto caab9; E5ecc: $privateKey = $rsaKeyGenerator->generatePrivateKey(); goto bbbd9; F3dce: $rsaKeyGenerator = new RsaKeyGenerator(); goto E5ecc; A5fc0: $siteEntity->setCertificate($certificateEntity); goto aeb30; D0201: $certificateEntity->setDefaultCertificate(true); goto fa09f; Eb34c: goto a18cc; goto Ddb1a; Ab426: $redirectionVhostTemplate = file_get_contents($redirectionVhostTemplateFile); goto fe9e0; D6603: $subjectAlternativeNames[] = sprintf("\167\167\167\56\x25\x73", $domainName); goto ee4cb; B98d7: $csrGenerator = new CsrGenerator($privateKey, $distinguishedName); goto F9295; Dc0ce: $siteEntity->setType(SiteEntity::TYPE_STATIC); goto A8cf4; C7654: $distinguishedName = new DistinguishedName($domainName, $subjectAlternativeNames); goto B98d7; ee759: $siteEntity->setVhostTemplate($vhostTemplate->getContent()); goto Eab51; B0a72: $resolvedDomainName = $this->domainNameParser->resolveDomainName($domainName); goto e1b7a; e45fe: $vhostTemplate->setContent($siteEntity->getVhostTemplate()); goto b1bc8; db8d0: } private function createReverseProxySite(string $domainName, string $reverseProxyUrl, string $siteUser, string $siteUserPassword) : SiteEntity { goto Fda4e; b5a86: $certificateEntity = $this->certificateEntityManager->createEntity(); goto F9962; a3100: $reverseProxySite->setUser($siteEntity->getUser()); goto a2e5b; C80f1: C6e25: goto b5a86; b9740: $siteConstraints = $this->validator->validate($siteEntity); goto B9db8; bea11: dc9c8: goto da40f; b15ff: $siteEntity = $this->siteEntityManager->createEntity(); goto D1b02; a539f: if (!(false === is_null($subdomain) && "\x77\x77\x77" == $subdomain)) { goto F5431; } goto D8e21; E70f3: $siteEntity->setUserPassword($siteUserPassword); goto Dc85a; A9279: $certificateEntity->setType(CertificateEntity::TYPE_SELF_SIGNED); goto d7273; Fe028: $siteEntity->setCertificate($certificateEntity); goto cd36c; E6c28: $vhostTemplate->addProcessor(new RedirectDomainProcessor()); goto e53cb; Ff43d: $reverseProxySite->setRegistrableDomain($registrableDomain); goto b975c; B390d: $constraintViolationList->addAll($siteConstraints); goto Ea611; B6a25: $selfSignedCertificate = Openssl::createSelfSignedCertificate($privateKey, $csr); goto bb46b; A7352: if (!(true === is_null($subdomain) || "\x77\167\x77" == $subdomain)) { goto d5e0e; } goto deecb; cd36c: $reverseProxySite = new ReverseProxySite(); goto a3100; bac8f: $subdomain = false === empty($subdomain) ? $subdomain : null; goto E11ed; Df645: $reverseProxySite->setReverseProxyUrl($reverseProxyUrl); goto a5160; E7d9c: $reversProxySiteCreator = new ReverseProxySiteCreator($reverseProxySite); goto bc213; a70f4: $siteEntity->setVhostTemplate($vhostTemplate->getContent()); goto d1be0; D1b02: $siteEntity->setType(SiteEntity::TYPE_REVERSE_PROXY); goto Cef13; c68f6: $privateKey = $rsaKeyGenerator->generatePrivateKey(); goto Ddece; E6db1: $reversProxySiteCreator->createRootDirectory(); goto D893c; c5218: $redirectionVhostTemplateFile = realpath(dirname(__FILE__) . "\x2f\56\56\57\x2e\x2e\x2f\56\x2e\x2f\162\x65\163\x6f\165\x72\x63\x65\163\x2f\x6e\x67\151\156\x78\x2f\166\x68\157\x73\164\137\164\x65\x6d\x70\154\141\x74\x65\x2f\162\145\x64\151\162\x65\143\x74\x2d\150\x74\x74\x70\63"); goto bea11; Af220: d5e0e: goto f8391; bb46b: $certificateEntity->setDefaultCertificate(true); goto A9279; D0d88: a3993: goto c5218; a5160: $reverseProxySite->setVhostTemplate($siteEntity->getVhostTemplate()); goto E7d9c; dbd82: $subdomain = $resolvedDomainName->subDomain()->toString(); goto bac8f; f8391: $siteEntity->setVhostTemplate($vhostTemplate); goto b9740; D893c: $reversProxySiteCreator->createLogrotateFile(); goto d8c4b; d7273: $certificateEntity->setCsr($csr); goto F724f; ccb54: $vhostTemplate->setContent($siteEntity->getVhostTemplate()); goto a1f85; bd7bc: f1410: goto Af220; D66d0: c511e: goto a539f; bc213: $reversProxySiteCreator->createUser(); goto E6db1; B9db8: if (!count($siteConstraints)) { goto C6e25; } goto B70c3; deecb: if (true === isset($_ENV["\101\120\120\x5f\x48\x54\x54\120\63"]) && "\x74\x72\165\145" === $_ENV["\x41\120\x50\x5f\110\124\124\120\63"]) { goto a3993; } goto E9704; D8e21: $subjectAlternativeNames[] = $registrableDomain; goto eccf0; Cef13: $siteEntity->setDomainName($domainName); goto C9835; a2e5b: $reverseProxySite->setUserPassword($siteEntity->getUserPassword()); goto e3ce0; Ddece: $subjectAlternativeNames = []; goto b8c9b; F724f: $certificateEntity->setPrivateKey($privateKey->getPEM()); goto b94af; Ea611: $errorMessages = $this->getConstraintViolationListErrorMessages($constraintViolationList); goto fad1b; eccf0: F5431: goto c3fb3; c0d4d: if (!(false === empty($redirectionVhostTemplate))) { goto f1410; } goto af217; d8c4b: $reversProxySiteCreator->createPrivateKeyAndCertificate(); goto Eaaae; b94af: $certificateEntity->setCertificate($selfSignedCertificate); goto Fe028; b8c9b: if (!(true === is_null($subdomain))) { goto c511e; } goto b053b; af217: $vhostTemplate = sprintf("\45\x73\45\x73", $redirectionVhostTemplate, $vhostTemplate); goto bd7bc; F9962: $certificateEntity->setSite($siteEntity); goto B61ef; e53cb: $vhostTemplate->build(); goto a70f4; B61ef: $rsaKeyGenerator = new RsaKeyGenerator(); goto c68f6; c0430: $reverseProxySite->setCertificate($certificateEntity); goto Df645; Bdd67: $vhostTemplate->addProcessor(new RedirectServerNameProcessor()); goto E6c28; f2c53: $registrableDomain = $resolvedDomainName->registrableDomain()->toString(); goto dbd82; E9704: $redirectionVhostTemplateFile = realpath(dirname(__FILE__) . "\x2f\56\56\x2f\x2e\56\x2f\56\56\57\162\145\163\x6f\165\162\x63\x65\163\57\x6e\147\x69\156\x78\x2f\166\150\x6f\163\x74\137\164\145\155\x70\x6c\141\x74\145\57\162\145\144\151\x72\145\x63\x74"); goto A6ae6; bf8ca: $vhostTemplate = new ReverseProxyVhostTemplate($reverseProxySite); goto ccb54; e2f44: $reversProxySiteCreator->resetPermissions(); goto bf8ca; d43e3: $vhostTemplate = $vhostTemplateEntity->getTemplate(); goto b15ff; Abafc: $csrGenerator = new CsrGenerator($privateKey, $distinguishedName); goto Ae485; Ae485: $csr = $csrGenerator->generate(); goto B6a25; e3ce0: $reverseProxySite->setDomainName($domainName); goto Ff43d; A6ae6: goto dc9c8; goto D0d88; Eaaae: $reversProxySiteCreator->createNginxVhost(); goto c6604; d1be0: $siteEntity->setApplication($vhostTemplateEntity->getName()); goto e10bf; dcb12: $siteEntity->setReverseProxyUrl($reverseProxyUrl); goto A7352; Dc85a: $siteEntity->setRootDirectory($domainName); goto dcb12; c6604: $reversProxySiteCreator->reloadNginxService(); goto e2f44; fad1b: throw new \Exception(implode("\x3b\40", $errorMessages)); goto C80f1; a291c: $vhostTemplate->addProcessor(new ServerNameProcessor()); goto Bdd67; C9835: $siteEntity->setUser($siteUser); goto E70f3; da40f: $redirectionVhostTemplate = file_get_contents($redirectionVhostTemplateFile); goto c0d4d; B70c3: $constraintViolationList = new ConstraintViolationList(); goto B390d; Fda4e: $resolvedDomainName = $this->domainNameParser->resolveDomainName($domainName); goto f2c53; e10bf: $this->siteEntityManager->updateEntity($siteEntity); goto fe853; b975c: $reverseProxySite->setSubdomain($subdomain); goto E244b; a1f85: $vhostTemplate->resetProcessors(); goto a291c; fe853: return $siteEntity; goto bba18; E11ed: $vhostTemplateEntity = $this->getVhostTemplateEntity(self::REVERSE_PROXY_VHOST_TEMPLATE_NAME); goto d43e3; c3fb3: $distinguishedName = new DistinguishedName($domainName, $subjectAlternativeNames); goto Abafc; E244b: $reverseProxySite->setRootDirectory($siteEntity->getRootDirectory()); goto c0430; b053b: $subjectAlternativeNames[] = sprintf("\167\x77\x77\x2e\45\163", $domainName); goto D66d0; bba18: } public function installedWordPressSite(Request $request) : Response { goto dea3b; ea04e: A9634: goto A03bd; D6f76: $response = $this->redirect($this->generateUrl("\143\x6c\160\137\163\151\x74\145\x73")); goto De898; Af77a: ef06a: goto dc88c; dea3b: $session = $request->getSession(); goto A6446; A03bd: $siteCredentials = $this->renderSiteCredentials($siteCredentials); goto b2a41; e37e9: if (false === empty($siteCredentials)) { goto A9634; } goto D6f76; dc88c: return $response; goto e51a7; b2a41: $response = $this->render("\x46\162\157\156\164\145\156\x64\x2f\123\x69\x74\145\x2f\116\145\167\57\167\x6f\x72\144\160\162\145\163\x73\55\151\156\x73\164\x61\x6c\x6c\x65\x64\56\x68\x74\x6d\x6c\56\x74\167\151\147", ["\x73\151\164\x65\x43\x72\x65\x64\x65\x6e\x74\x69\141\154\x73" => $siteCredentials]); goto Af929; Af929: return $response; goto Af77a; De898: goto ef06a; goto ea04e; A6446: $siteCredentials = $session->get("\163\151\x74\x65\x43\162\145\x64\145\156\x74\x69\141\x6c\x73"); goto e37e9; e51a7: } private function renderSiteCredentials(array $siteCredentials) : string { goto d3ed8; Beba1: foreach ($siteCredentials as $section => $sectionData) { goto f7bee; fefda: $renderedCredentials .= PHP_EOL; goto ea163; ea163: dc3cb: goto a40b8; a3cd9: fa36d: goto fefda; ea1a0: $renderedCredentials .= "\55\55\55\x2d\x2d\55\x2d\55\x2d\55\x2d\x2d\x2d\x2d\55\x2d\x2d\x2d\55\55\x2d\55\55\x2d\x2d\55\55\x2d\x2d\x2d\55\55\55\x2d\55\55\x2d\x2d\x2d\55\55\55\55\55\x2d\55\55\x2d\55\55\55\55\x2d\55" . PHP_EOL; goto cb91b; f7bee: $renderedCredentials .= $section . PHP_EOL; goto ea1a0; cb91b: foreach ($sectionData as $key => $value) { $renderedCredentials .= sprintf("\45\163\72\x20\x25\163", $key, $value) . PHP_EOL; C8a57: } goto a3cd9; a40b8: } goto D5bcc; d3ed8: $renderedCredentials = ''; goto Beba1; D5bcc: a2343: goto Ad417; Ad417: return $renderedCredentials; goto a606c; a606c: } private function getVhostTemplateEntity(string $name) : mixed { goto ba81a; Bf19b: if (!(true === is_null($vhostTemplateEntity))) { goto a853d; } goto Dbcc6; c23c6: a853d: goto e13e0; Dbcc6: throw new \Exception($this->translator->trans("\126\150\x6f\163\164\x20\x54\x65\155\x70\x6c\141\x74\x65\40\x25\x76\x68\x6f\163\x74\124\145\155\160\154\141\x74\145\45\40\144\x6f\x65\x73\x20\156\x6f\x74\x20\x65\x78\151\163\164\56", ["\x25\166\x68\157\x73\164\124\x65\x6d\x70\154\141\x74\x65\x25" => $name])); goto c23c6; e13e0: return $vhostTemplateEntity; goto Ed4ce; ba81a: $vhostTemplateEntity = $this->vhostTemplateEntityManager->findOneByName($name); goto Bf19b; Ed4ce: } private function getConstraintViolationListErrorMessages(ConstraintViolationList $constraintViolationList) : array { goto Cb36d; c3739: foreach ($constraintViolationList as $constraint) { $errorMessages[] = sprintf("\45\163\72\40\x25\163", $constraint->getPropertyPath(), $constraint->getMessage()); bae3b: } goto a362a; a362a: b554b: goto c8f08; c8f08: return $errorMessages; goto ad71e; Cb36d: $errorMessages = []; goto c3739; ad71e: } private function generateNameFromDomainName(string $domainName) : string { goto C8f37; a6440: F22a5: goto e04f1; A8a3a: e7efe: goto ce0de; ce0de: return $name; goto F51d1; d9800: f0aef: goto c8330; b0efa: $name = $explodedRegistrableDomain[0] ?? ''; goto B5003; F0861: $name = sprintf("\45\163\x2d\x25\x73", $explodedRegistrableDomain[0] ?? '', implode("\55", $explodedSubdomain)); goto d9800; c8330: goto e7efe; goto a6440; b9ab7: $subdomain = $resolvedDomainName->subDomain()->toString(); goto c1993; Bfdb3: if ("\x77\x77\167" == $subdomain) { goto F22a5; } goto b0efa; c1443: $registrableDomain = $resolvedDomainName->registrableDomain()->toString(); goto b9ab7; B5003: if (!(false === empty($subdomain))) { goto f0aef; } goto A7af3; e04f1: $name = $explodedRegistrableDomain[0] ?? ''; goto A8a3a; C8f37: $resolvedDomainName = $this->domainNameParser->resolveDomainName($domainName); goto c1443; A7af3: $explodedSubdomain = explode("\56", $subdomain); goto F0861; c1993: $explodedRegistrableDomain = explode("\56", $registrableDomain); goto Bfdb3; F51d1: } private function getUniqueDatabaseName(string $databaseName) : string { goto Bddb8; a2d0b: df600: goto Ddefd; ef428: $i = 0; goto Ecfc1; Bf8bc: $i++; goto Ae8cf; Ae8cf: if (true === $databaseExists) { goto df600; } goto F7ddd; C9296: return $uniqueDatabaseName; goto fac1b; Ddefd: if (!(0 < $i)) { goto d940d; } goto D37ac; ea777: b64de: goto ef428; Dabcf: d940d: goto cf1a4; d9a3a: $databases = $databaseConnection->getDatabases(); goto a2d0b; Bd935: if (!(strlen($uniqueDatabaseName) <= 3)) { goto b64de; } goto c0792; c0792: $uniqueDatabaseName = sprintf("\45\x73\x2d\144\x62", $uniqueDatabaseName); goto ea777; F7ddd: Dfb27: goto C9296; cf1a4: $databaseExists = in_array($uniqueDatabaseName, $databases); goto Bf8bc; Bddb8: $uniqueDatabaseName = $databaseName; goto Bd935; D37ac: $uniqueDatabaseName = sprintf("\45\163\x2d\45\x73", $databaseName, $i); goto Dabcf; bd39d: $databaseConnection = new DatabaseConnection($activeDatabaseServerEntity); goto d9a3a; Ecfc1: $activeDatabaseServerEntity = $this->databaseServerEntityManager->getActiveDatabaseServer(); goto bd39d; fac1b: } private function getUniqueDatabaseUserName(string $databaseUserName) : string { goto A1f73; ea2ab: return $uniqueDatabaseUserName; goto Ef712; fe325: Fb073: goto A0c92; Ccbf2: if (!(0 < $i)) { goto E9720; } goto D4cdf; Ba7d5: $databaseUserEntity = $this->databaseUserEntityManager->findOneByUserName($uniqueDatabaseUserName); goto b332a; Ff842: $uniqueDatabaseUserName = sprintf("\45\x73\x2d\x64\142", $uniqueDatabaseUserName); goto fe325; ad222: if (!(strlen($uniqueDatabaseUserName) <= 3)) { goto Fb073; } goto Ff842; C089e: E9720: goto Ba7d5; a2b73: if (false === is_null($databaseUserEntity)) { goto F45ab; } goto C3b5e; Edf3a: $uniqueDatabaseUserName = mb_substr($uniqueDatabaseUserName, 0, 32); goto ea2ab; b332a: $i++; goto a2b73; A0c92: $i = 0; goto ab07b; ab07b: F45ab: goto Ccbf2; D4cdf: $uniqueDatabaseUserName = sprintf("\x25\x73\55\x25\x73", $databaseUserName, $i); goto C089e; C3b5e: B1a20: goto Edf3a; A1f73: $uniqueDatabaseUserName = $databaseUserName; goto ad222; Ef712: } private function hasPermissions() : bool { goto B285f; aa9c1: return false; goto f4f18; f4903: if (!(UserEntity::ROLE_USER != $user->getRole())) { goto e3c15; } goto f865f; B285f: $user = $this->getUser(); goto f4903; f865f: return true; goto Adac8; Adac8: e3c15: goto aa9c1; f4f18: } }
